import logging
import fmcapi
import time


def test__devicerecords(fmc):
    logging.info('Test Device.  Though you can "Post" devices I do not have one handy. So '
                 'add/remove licenses on Device Objects.')

    starttime = str(int(time.time()))
    namer = f'_fmcapi_test_{starttime}'

    acp1 = fmcapi.AccessPolicies(fmc=fmc, name=namer)
    acp1.post()
    obj1 = fmcapi.DeviceRecords(fmc=fmc)
    obj1.name = namer
    obj1.acp(name=acp1.name)
    obj1.licensing(action='add', name='MALWARE')
    obj1.licensing(action='add', name='VPN')
    obj1.licensing(action='remove', name='VPN')
    obj1.licensing(action='clear')
    obj1.licensing(action='add', name='BASE')
    logging.info('Device -->')
    logging.info(obj1.format_data())

    acp1.delete()

    logging.info('Test Device done.\n')



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
from .ftdnatpolicies import FTDNatPolicies
from fmcapi.api_objects.object_services.networkaddresses import NetworkAddresses
from fmcapi.api_objects.object_services.networkgroups import NetworkGroups
from fmcapi.api_objects.object_services.portobjectgroups import PortObjectGroups
from fmcapi.api_objects.object_services.protocolportobjects import ProtocolPortObjects
from fmcapi.api_objects.object_services.interfaceobjects import InterfaceObjects
import logging


class ManualNatRules(APIClassTemplate):
    # Host,Network,NetworkGroup objects
    """
    The ManualNatRules Object in the FMC.
    """

    VALID_JSON_DATA = ['id', 'name', 'type', 'originalSource', 'originalDestination', 'translatedSource',
                       'translatedDestination', 'interfaceInTranslatedSource', 'interfaceInOriginalDestination',
                       'natType', 'interfaceIpv6', 'fallThrough', 'dns', 'routeLookup', 'noProxyArp', 'netToNet',
                       'sourceInterface', 'destinationInterface', 'originalSourcePort', 'translatedSourcePort',
                       'originalDestinationPort', 'translatedDestinationPort', 'patOptions', 'unidirectional',
                       'enabled',
                       ]
    VALID_FOR_KWARGS = VALID_JSON_DATA + []
    PREFIX_URL = '/policy/ftdnatpolicies'
    REQUIRED_FOR_POST = ["nat_id"]

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for ManualNatRules class.")
        self.parse_kwargs(**kwargs)
        self.type = "FTDManualNatRule"

    def parse_kwargs(self, **kwargs):
        super().parse_kwargs(**kwargs)
        logging.debug("In parse_kwargs() for ManualNatRules class.")
        if 'translatedSource' in kwargs and 'interfaceInTranslatedSource' is True:
            logging.warning("Cannot have both a translatedSource and interfaceInTranslatedSource")
        elif 'translatedSource' in kwargs:
            self.translatedSource = kwargs['translatedSource']
        elif 'interfaceInTranslatedSource' in kwargs:
            self.interfaceInTranslatedSource = kwargs['interfaceInTranslatedSource']

    def nat_policy(self, name):
        logging.debug("In nat_policy() for ManualNatRules class.")
        ftd_nat = FTDNatPolicies(fmc=self.fmc)
        ftd_nat.get(name=name)
        if 'id' in ftd_nat.__dict__:
            self.nat_id = ftd_nat.id
            self.URL = f'{self.fmc.configuration_url}{self.PREFIX_URL}/{self.nat_id}/manualnatrules'
            self.nat_added_to_url = True
        else:
            logging.warning(f'FTD NAT Policy "{name}" not found.  Cannot set up ManualNatRule for NAT Policy.')

    def original_source(self, name):
        logging.debug("In original_source() for ManualNatRules class.")
        ipaddresses_json = NetworkAddresses(fmc=self.fmc).get()
        networkgroup_json = NetworkGroups(fmc=self.fmc).get()
        items = ipaddresses_json.get('items', [])
        new_net = None
        for item in items:
            if item['name'] == name:
                new_net = {'id': item['id'], 'type': item['type']}
                break
        if new_net is None:
            logging.warning(f'Network "{name}" is not found in FMC.  Cannot add to original_source.')
        else:
            self.originalSource = new_net
            logging.info(f'Adding "{name}" to original_source for this ManualNatRule.')

    def translated_source(self, name):
        logging.debug("In translated_source() for ManualNatRules class.")
        ipaddresses_json = NetworkAddresses(fmc=self.fmc).get()
        networkgroup_json = NetworkGroups(fmc=self.fmc).get()
        items = ipaddresses_json.get('items', [])
        new_net = None
        for item in items:
            if item['name'] == name:
                new_net = {'id': item['id'], 'type': item['type']}
                break
        if new_net is None:
            logging.warning(f'Network "{name}" is not found in FMC.  Cannot add to translated_source.')
        else:
            self.translatedSource = new_net
            logging.info(f'Adding "{name}" to translated_source for this ManualNatRule.')

    def original_destination(self, name):
        logging.debug("In original_destination() for ManualNatRules class.")
        ipaddresses_json = NetworkAddresses(fmc=self.fmc).get()
        networkgroup_json = NetworkGroups(fmc=self.fmc).get()
        items = ipaddresses_json.get('items', [])
        new_net = None
        for item in items:
            if item['name'] == name:
                new_net = {'id': item['id'], 'type': item['type']}
                break
        if new_net is None:
            logging.warning(f'Network "{name}" is not found in FMC.  Cannot add to original_destination.')
        else:
            self.originalDestination = new_net
            logging.info(f'Adding "{name}" to original_destination for this ManualNatRule.')

    def translated_destination(self, name):
        logging.debug("In translated_destination() for ManualNatRules class.")
        ipaddresses_json = NetworkAddresses(fmc=self.fmc).get()
        networkgroup_json = NetworkGroups(fmc=self.fmc).get()
        items = ipaddresses_json.get('items', [])
        new_net = None
        for item in items:
            if item['name'] == name:
                new_net = {'id': item['id'], 'type': item['type']}
                break
        if new_net is None:
            logging.warning(f'Network "{name}" is not found in FMC.  Cannot add to translated_destination.')
        else:
            self.translatedDestination = new_net
            logging.info(f'Adding "{name}" to translated_destination for this ManualNatRule.')

    def original_source_port(self, name):
        logging.debug("In original_source_port() for ManualNatRules class.")
        ports_json = ProtocolPortObjects(fmc=self.fmc).get()
        portgroup_json = PortObjectGroups(fmc=self.fmc).get()
        items = ports_json.get('items', []) + portgroup_json.get('items', [])
        new_port = None
        for item in items:
            if item['name'] == name:
                new_port = {'id': item['id'], 'type': item['type']}
                break
        if new_port is None:
            logging.warning(f'Port "{name}" is not found in FMC.  Cannot add to original_source_port.')
        else:
            self.originalSourcePort = new_port
            logging.info(f'Adding "{name}" to original_source_port for this ManualNatRule.')

    def translated_source_port(self, name):
        logging.debug("In translated_source_port() for ManualNatRules class.")
        ports_json = ProtocolPortObjects(fmc=self.fmc).get()
        portgroup_json = PortObjectGroups(fmc=self.fmc).get()
        items = ports_json.get('items', []) + portgroup_json.get('items', [])
        new_port = None
        for item in items:
            if item['name'] == name:
                new_port = {'id': item['id'], 'type': item['type']}
                break
        if new_port is None:
            logging.warning(f'Port "{name}" is not found in FMC.  Cannot add to translated_source_port.')
        else:
            self.translatedSourcePort = new_port
            logging.info(f'Adding "{name}" to translated_source_port for this ManualNatRule.')

    def original_destination_port(self, name):
        logging.debug("In original_destination_port() for ManualNatRules class.")
        ports_json = ProtocolPortObjects(fmc=self.fmc).get()
        portgroup_json = PortObjectGroups(fmc=self.fmc).get()
        items = ports_json.get('items', []) + portgroup_json.get('items', [])
        new_port = None
        for item in items:
            if item['name'] == name:
                new_port = {'id': item['id'], 'type': item['type']}
                break
        if new_port is None:
            logging.warning(f'Port "{name}" is not found in FMC.  Cannot add to original_destination_port.')
        else:
            self.originalDestinationPort = new_port
            logging.info(f'Adding "{name}" to original_destination_port for this ManualNatRule.')

    def translated_destination_port(self, name):
        logging.debug("In translated_destination_port() for ManualNatRules class.")
        ports_json = ProtocolPortObjects(fmc=self.fmc).get()
        portgroup_json = PortObjectGroups(fmc=self.fmc).get()
        items = ports_json.get('items', []) + portgroup_json.get('items', [])
        new_port = None
        for item in items:
            if item['name'] == name:
                new_port = {'id': item['id'], 'type': item['type']}
                break
        if new_port is None:
            logging.warning(f'Port "{name}" is not found in FMC.  Cannot add to translated_destination_port.')
        else:
            self.translatedDestinationPort = new_port
            logging.info(f'Adding "{name}" to translated_destination_port for this ManualNatRule.')

    def source_intf(self, name):
        logging.debug("In source_intf() for ManualNatRules class.")
        intf_obj = InterfaceObjects(fmc=self.fmc).get()
        items = intf_obj.get('items', [])
        new_intf = None
        for item in items:
            if item["name"] == name:
                new_intf = {'id': item['id'], 'type': item['type']}
                break
        if new_intf is None:
            logging.warning(f'Interface Object "{name}" is not found in FMC.  Cannot add to sourceInterface.')
        else:
            self.sourceInterface = new_intf
            logging.info(f'Interface Object "{name}" added to NAT Policy.')

    def destination_intf(self, name):
        logging.debug("In destination_intf() for ManualNatRules class.")
        intf_obj = InterfaceObjects(fmc=self.fmc).get()
        items = intf_obj.get('items', [])
        new_intf = None
        for item in items:
            if item["name"] == name:
                new_intf = {'id': item['id'], 'type': item['type']}
                break
        if new_intf is None:
            logging.warning(f'Interface Object "{name}" is not found in FMC.  Cannot add to destinationInterface.')
        else:
            self.destinationInterface = new_intf
            logging.info(f'Interface Object "{name}" added to NAT Policy.')

    def identity_nat(self, name):
        logging.debug("In identity_nat() for ManualNatRules class.")
        ipaddresses_json = NetworkAddresses(fmc=self.fmc).get()
        networkgroup_json = NetworkGroups(fmc=self.fmc).get()
        items = ipaddresses_json.get('items', []) + networkgroup_json.get('items', [])
        new_net = None
        for item in items:
            if item['name'] == name:
                new_net = {'id': item['id'], 'type': item['type']}
                break
        if new_net is None:
            logging.warning(f'Network "{name}" is not found in FMC.  Cannot add to this ManualNatRules.')
        else:
            self.natType = "STATIC"
            self.originalSource = new_net
            self.translatedSource = new_net
            logging.info(f'Adding "{name}" to ManualNatRules.')

    def patPool(self, name, options={}):
        ipaddresses_json = NetworkAddresses(fmc=self.fmc).get()
        networkgroup_json = NetworkGroups(fmc=self.fmc).get()
        items = ipaddresses_json.get('items', []) + networkgroup_json.get('items', [])
        new_net = None
        for item in items:
            if item['name'] == name:
                new_net = {'name': item['name'], 'id': item['id'], 'type': item['type']}
                break
        if new_net is None:
            logging.warning(f'Network "{name}" is not found in FMC.  Cannot add to patPool.')
        else:
            self.natType = "DYNAMIC"
            self.unidirectional = True
            self.patOptions = {"patPoolAddress": new_net}
            self.patOptions["interfacePat"] = options.interfacePat if "interfacePat" in options.keys() else False
            self.patOptions["includeReserve"] = options.includeReserve if "includeReserve" in options.keys() else False
            self.patOptions["roundRobin"] = options.roundRobin if "roundRobin" in options.keys() else True
            self.patOptions["extendedPat"] = options.extendedPat if "extendedPat" in options.keys() else False
            self.patOptions["flatPortRange"] = options.flatPortRange if "flatPortRange" in options.keys() else False
            logging.info(f'Adding "{name}" to patPool for this ManualNatRule.')



import logging


def test__deployabledevices(fmc):
    logging.info('Testing fmc.deployabledevices() method.')
    logging.info(fmc.deployabledevices())
    logging.info('Testing fmc.deployabledevices() method done.\n')



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
from .devicerecords import DeviceRecords
import logging


class StaticRoutes(APIClassTemplate):
    """
    The StaticRoutes Object in the FMC.
    """

    VALID_JSON_DATA = ['id', 'name', 'type', 'continentId', 'continents', 'countries', 'continentUUID']
    VALID_FOR_KWARGS = VALID_JSON_DATA + []
    PREFIX_URL = '/devices/devicerecords'
    URL_SUFFIX = None

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for StaticRoutes class.")
        self.parse_kwargs(**kwargs)

    def device(self, device_name):
        logging.debug("In device() for StaticRoutes class.")
        device1 = DeviceRecords(fmc=self.fmc)
        device1.get(name=device_name)
        if 'id' in device1.__dict__:
            self.device_id = device1.id
            self.URL = f'{self.fmc.configuration_url}{self.PREFIX_URL}/{self.device_id}/routing/staticroutes'
            self.device_added_to_url = True
        else:
            logging.warning(f'Device "{device_name}" not found.  Cannot set up device for physicalInterface.')

    def post(self):
        logging.info('POST method for API for StaticRoutes not supported.')
        pass

    def put(self):
        logging.info('PUT method for API for StaticRoutes not supported.')
        pass

    def delete(self):
        logging.info('DELETE method for API for StaticRoutes not supported.')
        pass



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
from .ports import Ports
import logging
import warnings


class PortObjectGroups(APIClassTemplate):
    """
    The PortObjectGroups Object in the FMC.
    """

    VALID_JSON_DATA = ['id', 'name', 'type', 'objects', 'literals']
    VALID_FOR_KWARGS = VALID_JSON_DATA + []
    URL_SUFFIX = '/object/portobjectgroups'

    # Technically you can have objects OR literals but I'm not set up for "OR" logic, yet.
    REQUIRED_FOR_POST = ['name', 'objects']

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for PortObjectGroups class.")
        self.parse_kwargs(**kwargs)
        self.type = 'NetworkGroup'

    def named_ports(self, action, name=''):
        logging.debug("In named_ports() for PortObjectGroups class.")
        if action == 'add':
            port1 = Ports(fmc=self.fmc)
            response = port1.get()
            if 'items' in response:
                new_port = None
                for item in response['items']:
                    if item['name'] == name:
                        new_port = {'name': item['name'], 'id': item['id'], 'type': item['type']}
                        break
                if new_port is None:
                    logging.warning(f'Port "{name}" is not found in FMC.  Cannot add to PortObjectGroups.')
                else:
                    if 'objects' in self.__dict__:
                        duplicate = False
                        for obj in self.objects:
                            if obj['name'] == new_port['name']:
                                duplicate = True
                                break
                        if not duplicate:
                            self.objects.append(new_port)
                            logging.info(f'Adding "{name}" to PortObjectGroups.')
                    else:
                        self.objects = [new_port]
                        logging.info(f'Adding "{name}" to PortObjectGroups.')
        elif action == 'remove':
            if 'objects' in self.__dict__:
                objects_list = []
                for obj in self.objects:
                    if obj['name'] != name:
                        objects_list.append(obj)
                self.objects = objects_list
                logging.info(f'Removed "{name}" from PortObjectGroups.')
            else:
                logging.info("This PortObjectGroups has no named_ports.  Nothing to remove.")
        elif action == 'clear':
            if 'objects' in self.__dict__:
                del self.objects
                logging.info('All named_ports removed from this PortObjectGroups.')


class PortObjectGroup(PortObjectGroups):
    """Dispose of this Class after 20210101."""

    def __init__(self, fmc, **kwargs):
        warnings.resetwarnings()
        warnings.warn("Deprecated: PortObjectGroup() should be called via PortObjectGroups().")
        super().__init__(fmc, **kwargs)



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
from .accesspolicies import AccessPolicies
from fmcapi.api_objects.device_services.devicerecords import DeviceRecords
from .prefilterpolicies import PreFilterPolicies
import logging
import warnings


class HitCounts(APIClassTemplate):
    """
    The HitCounts Object in the FMC.
    """

    VALID_JSON_DATA = []
    VALID_FOR_KWARGS = VALID_JSON_DATA + ['acp_id', 'acp_name', 'device_id', 'device_name', 'fetchZeroHitcount',
                                          'limit',
                                          ]
    PREFIX_URL = '/policy/accesspolicies'
    REQUIRED_FOR_PUT = ['acp_id', 'device_id']
    REQUIRED_FOR_DELETE = ['acp_id', 'device_id']
    REQUIRED_FOR_GET = ['acp_id', 'device_id']
    VALID_CHARACTERS_FOR_NAME = """[.\w\d_\- ]"""
    FIRST_SUPPORTED_FMC_VERSION = '6.4'

    @property
    def URL_SUFFIX(self):
        """
        Add the URL suffixes for filter.
        """
        filter_init = '?filter="'
        filter_string = filter_init

        self.URL = self.URL.split('?')[0]

        if self.device_id:
            filter_string += f'deviceId:{self.device_id};'
        if self.prefilter_ids:
            filter_string += f'ids:{self.prefilter_ids};'
        if self.fetchZeroHitCount:
            filter_string += f'fetchZeroHitCount:{self._fetchZeroHitCount};'

        if filter_string == filter_init:
            filter_string += '"'
        filter_string = f'{filter_string[:-1]}"&expanded=true'

        if 'limit' in self.__dict__:
            filter_string += f'&limit={self.limit}'
        return filter_string

    @property
    def fetchZeroHitCount(self):
        return self._fetchZeroHitCount

    @fetchZeroHitCount.setter
    def fetchZeroHitCount(self, value=False):
        self._fetchZeroHitCount = value
        # Rebuild the URL with possible new information
        self.URL = self.URL.split('?')[0]
        self.URL = f'{self.URL}{self.URL_SUFFIX}'

    def __init__(self, fmc, **kwargs):
        logging.debug("In __init__() for HitCounts class.")
        self.device_id = None
        self.prefilter_ids = []
        self.fetchZeroHitCount = False
        super().__init__(fmc, **kwargs)
        self.parse_kwargs(**kwargs)
        self.type = 'HitCount'
        self.URL = f'{self.URL}{self.URL_SUFFIX}'

    def parse_kwargs(self, **kwargs):
        super().parse_kwargs(**kwargs)
        logging.debug("In parse_kwargs() for HitCounts class.")
        if 'acp_id' in kwargs:
            self.acp(acp_id=kwargs['acp_id'])
        if 'acp_name' in kwargs:
            self.acp(name=kwargs['acp_name'])
        if 'device_id' in kwargs:
            self.device(id=kwargs['device_id'])
        if 'device_name' in kwargs:
            self.device(name=kwargs['device_name'])

    def acp(self, name='', acp_id=''):
        # either name or id of the ACP should be given
        logging.debug("In acp() for HitCounts class.")
        if acp_id != '':
            self.acp_id = acp_id
            self.URL = f'{self.fmc.configuration_url}{self.PREFIX_URL}/{self.acp_id}/operational/hitcounts'
            self.acp_added_to_url = True
        elif name != '':
            acp1 = AccessPolicies(fmc=self.fmc)
            acp1.get(name=name)
            if 'id' in acp1.__dict__:
                self.acp_id = acp1.id
                self.URL = f'{self.fmc.configuration_url}{self.PREFIX_URL}/{self.acp_id}/operational/hitcounts'
                self.acp_added_to_url = True
            else:
                logging.warning(f'Access Control Policy "{name}" not found.  Cannot configure acp for HitCounts.')
        else:
            logging.error('No accessPolicy name or id was provided.')
        # Rebuild the URL with possible new information
        self.URL = self.URL.split('?')[0]
        self.URL = f'{self.URL}{self.URL_SUFFIX}'

    def device(self, name='', id=''):
        logging.debug("In device() for HitCounts class")
        if id != '':
            self.device_id = id
        elif name != '':
            device1 = DeviceRecords(fmc=self.fmc)
            device1.get(name=name)
            if 'id' in device1.__dict__:
                self.device_id = device1.id
            else:
                logging.warning(f'Device "{name}" not found.  Cannot configure device for HitCounts.')
        else:
            logging.error('No device name or id was provided.')
        # Rebuild the URL with possible new information
        self.URL = self.URL.split('?')[0]
        self.URL = f'{self.URL}{self.URL_SUFFIX}'

    def prefilter_policies(self, action, name='', prefilter_id=''):
        logging.debug("In prefilter_policies_tags() for HitCounts class.")
        if action == 'add':
            ppolicy = PreFilterPolicies(fmc=self.fmc)
            if prefilter_id:
                ppolicy.get(id=prefilter_id)
            elif name:
                ppolicy.get(name=name)
            if 'id' in ppolicy.__dict__:
                if self.prefilter_ids:
                    duplicate = False
                    for obj in self.prefilter_ids.split(','):
                        if obj == ppolicy.id:
                            duplicate = True
                            logging.warning(f'Id, {ppolicy.id}, already in prefilter_ids not duplicating.')
                            break
                    if not duplicate:
                        self.prefilter_ids.append(ppolicy.id)
                        logging.info(f'Adding "{ppolicy.id}" to prefilter_ids for this HitCounts.')
                else:
                    self.prefilter_ids.append(ppolicy.id)
                    logging.info(f'Adding "{ppolicy.id}" to prefilter_ids for this HitCounts.')
            else:
                if name:
                    logging.warning(f'Prefilter, "{name}", not found.  Cannot add to HitCounts.')
                elif prefilter_id:
                    logging.warning(f'Prefilter, {prefilter_id}, not found.  Cannot add to HitCounts.')
        elif action == 'remove':
            ppolicy = PreFilterPolicies(fmc=self.fmc)
            if prefilter_id:
                ppolicy.get(id=prefilter_id)
            elif name:
                ppolicy.get(name=name)
            if 'id' in ppolicy.__dict__:
                if self.prefilter_ids:
                    objects = []
                    for obj in self.prefilter_ids:
                        if obj != ppolicy.id:
                            objects.append(obj)
                    self.prefilter_ids = objects
                    logging.info(f'Removed "{ppolicy.id}" from prefilter_ids for this HitCounts.')
                else:
                    logging.info("prefilter_ids is empty for this HitCounts.  Nothing to remove.")
            else:
                logging.warning(f'Prefilter Policy, {ppolicy.id}, not found.  Cannot remove from HitCounts.')
        elif action == 'clear':
            if self.prefilter_ids:
                self.prefilter_ids = []
                logging.info('All prefilter_ids removed from this HitCounts object.')

        # Rebuild the URL with possible new information
        self.URL = self.URL.split('?')[0]
        self.URL = f'{self.URL}{self.URL_SUFFIX}'

    def get(self, **kwargs):
        """
        Get HitCounts based on filter criteria
        :return:
        """
        logging.debug("In get() for HitCount class.")
        self.parse_kwargs(**kwargs)
        if self.fmc.serverVersion < self.FIRST_SUPPORTED_FMC_VERSION:
            logging.error(f'Your FMC version, {self.fmc.serverVersion} does not support GET of this feature.')
            return {'items': []}
        if self.valid_for_get() and (self.device_id or self.prefilter_ids):
            if self.dry_run:
                logging.info('Dry Run enabled.  Not actually sending to FMC.  Here is what would have been sent:')
                logging.info('\tMethod = GET')
                logging.info(f'\tURL = {self.URL}')
                return False
            response = self.fmc.send_to_api(method='get', url=self.URL)
            self.parse_kwargs(**response)
            if 'items' not in response:
                response['items'] = []
            return response
        else:
            logging.warning("get() method failed due to failure to pass valid_for_get() test.")
            return False

    def put(self, **kwargs):
        logging.info('Though supported by FMC, API PUT method is not yet working for HitCounts in fmcapi.')
        pass

    def delete(self, **kwargs):
        logging.info('Though supported by FMC, API DELETE method is not yet working for HitCounts in fmcapi.')
        pass

    def post(self):
        logging.info('API POST method for HitCounts not supported.')
        pass


class HitCount(HitCounts):
    """Dispose of this Class after 20210101."""

    def __init__(self, fmc, **kwargs):
        warnings.resetwarnings()
        warnings.warn("Deprecated: HitCount() should be called via HitCount().")
        super().__init__(fmc, **kwargs)



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
from fmcapi.api_objects.policy_services.accesspolicies import AccessPolicies
from fmcapi.api_objects.policy_services.intrusionpolicies import IntrusionPolicies
from fmcapi.api_objects.object_services.variablesets import VariableSets
from fmcapi.api_objects.object_services.securityzones import SecurityZones
from fmcapi.api_objects.object_services.vlantags import VlanTags
from fmcapi.api_objects.object_services.portobjectgroups import PortObjectGroups
from fmcapi.api_objects.object_services.protocolportobjects import ProtocolPortObjects
from fmcapi.api_objects.object_services.fqdns import FQDNS
from fmcapi.api_objects.object_services.networkgroups import NetworkGroups
from fmcapi.api_objects.object_services.networkaddresses import NetworkAddresses
from fmcapi.api_objects.policy_services.filepolicies import FilePolicies
from fmcapi.api_objects.object_services.isesecuritygrouptags import ISESecurityGroupTags
from fmcapi.api_objects.helper_functions import get_networkaddress_type
import logging
import sys
import warnings


class AccessRules(APIClassTemplate):
    """
    The AccessRules Object in the FMC.
    """
    VALID_JSON_DATA = ['id', 'name', 'type', 'action', 'enabled', 'sendEventsToFMC', 'logFiles', 'logBegin', 'logEnd',
                       'variableSet', 'originalSourceNetworks', 'vlanTags', 'sourceNetworks', 'destinationNetworks',
                       'sourcePorts', 'destinationPorts', 'ipsPolicy', 'urls', 'sourceZones', 'destinationZones',
                       'applications', 'filePolicy', 'sourceSecurityGroupTags', 'destinationSecurityGroupTags',
                       ]
    VALID_FOR_KWARGS = VALID_JSON_DATA + ['acp_id', 'acp_name', 'insertBefore', 'insertAfter', 'section']
    PREFIX_URL = '/policy/accesspolicies'
    REQUIRED_FOR_POST = ['name', 'acp_id']
    REQUIRED_FOR_GET = ['acp_id']
    VALID_FOR_ACTION = ['ALLOW', 'TRUST', 'BLOCK', 'MONITOR', 'BLOCK_RESET', 'BLOCK_INTERACTIVE',
                        'BLOCK_RESET_INTERACTIVE']
    VALID_CHARACTERS_FOR_NAME = """[.\w\d_\- ]"""

    @property
    def URL_SUFFIX(self):
        """
        Add the URL suffixes for categories, insertBefore and insertAfter
        NOTE: You must specify these at the time the object is initialized (created) for this feature
        to work correctly. Example:
            This works:
                new_rule = AccessRules(fmc=fmc, acp_name='acp1', insertBefore=2)

            This does not:
                new_rule = AccessRules(fmc=fmc, acp_name='acp1')
                new_rule.insertBefore = 2
        """
        url = '?'

        if 'category' in self.__dict__:
            url = f'{url}category={self.category}&'
        if 'insertBefore' in self.__dict__:
            url = f'{url}insertBefore={self.insertBefore}&'
        if 'insertAfter' in self.__dict__:
            url = f'{url}insertAfter={self.insertAfter}&'
        if 'insertBefore' in self.__dict__ and 'insertAfter' in self.__dict__:
            logging.warning('ACP rule has both insertBefore and insertAfter params')
        if 'section' in self.__dict__:
            url = f'{url}section={self.section}&'

        return url[:-1]

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for AccessRules class.")
        self.type = 'AccessRule'
        self.parse_kwargs(**kwargs)
        self.URL = f'{self.URL}{self.URL_SUFFIX}'

    def format_data(self):
        json_data = super().format_data()
        logging.debug("In format_data() for AccessRules class.")
        if 'sourceNetworks' in self.__dict__:
            json_data['sourceNetworks'] = {'objects': self.sourceNetworks['objects']}
            json_data['sourceNetworks']['literals'] = \
                [{'type': v, 'value': k} for k, v in self.sourceNetworks['literals'].items()]
        if 'destinationNetworks' in self.__dict__:
            json_data['destinationNetworks'] = {'objects': self.destinationNetworks['objects']}
            json_data['destinationNetworks']['literals'] = \
                [{'type': v, 'value': k} for k, v in self.destinationNetworks['literals'].items()]
        if 'action' in self.__dict__:
            if self.action not in self.VALID_FOR_ACTION:
                logging.warning(f"Action {self.action} is not a valid action.")
                logging.warning(f"\tValid actions are: {self.VALID_FOR_ACTION}.")
        return json_data

    def parse_kwargs(self, **kwargs):
        super().parse_kwargs(**kwargs)
        logging.debug("In parse_kwargs() for AccessRules class.")
        if 'acp_id' in kwargs:
            self.acp(acp_id=kwargs['acp_id'])
        if 'acp_name' in kwargs:
            self.acp(name=kwargs['acp_name'])
        if 'action' in kwargs:
            if kwargs['action'] in self.VALID_FOR_ACTION:
                self.action = kwargs['action']
            else:
                logging.warning(f"Action {kwargs['action']} is not a valid action.")
                logging.warning(f"\tValid actions are: {self.VALID_FOR_ACTION}.")
        if 'sourceNetworks' in kwargs:
            self.sourceNetworks = {'objects': [], 'literals': {}}
            if kwargs['sourceNetworks'].get('objects'):
                self.sourceNetworks['objects'] = kwargs['sourceNetworks']['objects']
            if kwargs['sourceNetworks'].get('literals'):
                for literal in kwargs['sourceNetworks']['literals']:
                    self.sourceNetworks['literals'][literal['value']] = literal['type']
        if 'destinationNetworks' in kwargs:
            self.destinationNetworks = {'objects': [], 'literals': {}}
            if kwargs['destinationNetworks'].get('objects'):
                self.destinationNetworks['objects'] = kwargs['destinationNetworks']['objects']
            if kwargs['destinationNetworks'].get('literals'):
                for literal in kwargs['destinationNetworks']['literals']:
                    self.destinationNetworks['literals'][literal['value']] = literal['type']

        # Check if suffix should be added to URL
        # self.url_suffix()

    def acp(self, name='', acp_id=''):
        # either name or id of the ACP should be given
        logging.debug("In acp() for AccessRules class.")
        if acp_id != '':
            self.acp_id = acp_id
            self.URL = f'{self.fmc.configuration_url}{self.PREFIX_URL}/{self.acp_id}/accessrules'
            self.acp_added_to_url = True
        elif name != '':
            acp1 = AccessPolicies(fmc=self.fmc)
            acp1.get(name=name)
            if 'id' in acp1.__dict__:
                self.acp_id = acp1.id
                self.URL = f'{self.fmc.configuration_url}{self.PREFIX_URL}/{self.acp_id}/accessrules'
                self.acp_added_to_url = True
            else:
                logging.warning(f'Access Control Policy {name} not found.  Cannot set up accessPolicy for AccessRules.')
        else:
            logging.error('No accessPolicy name or ID was provided.')

    def intrusion_policy(self, action, name=''):
        logging.debug("In intrusion_policy() for AccessRules class.")
        if action == 'clear':
            if 'ipsPolicy' in self.__dict__:
                del self.ipsPolicy
                logging.info('Intrusion Policy removed from this AccessRules object.')
        elif action == 'set':
            ips = IntrusionPolicies(fmc=self.fmc, name=name)
            ips.get()
            self.ipsPolicy = {'name': ips.name, 'id': ips.id, 'type': ips.type}
            logging.info(f'Intrusion Policy set to "{name}" for this AccessRules object.')

    def variable_set(self, action, name='Default-Set'):
        logging.debug("In variable_set() for AccessRules class.")
        if action == 'clear':
            if 'variableSet' in self.__dict__:
                del self.variableSet
                logging.info('Variable Set removed from this AccessRules object.')
        elif action == 'set':
            vs = VariableSets(fmc=self.fmc)
            vs.get(name=name)
            self.variableSet = {'name': vs.name, 'id': vs.id, 'type': vs.type}
            logging.info(f'VariableSet set to "{name}" for this AccessRules object.')

    def file_policy(self, action, name='None'):
        logging.debug("In file_policy() for ACPRule class.")
        if action == 'clear':
            if 'filePolicy' in self.__dict__:
                del self.filePolicy
                logging.info('file_policy removed from this AccessRules object.')
        elif action == 'set':
            fp = FilePolicies(fmc=self.fmc)
            fp.get(name=name)
            self.filePolicy = {'name': fp.name, 'id': fp.id, 'type': fp.type}
            logging.info(f'file_policy set to "{name}" for this AccessRules object.')

    def vlan_tags(self, action, name=''):
        logging.debug("In vlan_tags() for AccessRules class.")
        if action == 'add':
            vlantag = VlanTags(fmc=self.fmc)
            vlantag.get(name=name)
            if 'id' in vlantag.__dict__:
                if 'vlanTags' in self.__dict__:
                    new_vlan = {'name': vlantag.name, 'id': vlantag.id, 'type': vlantag.type}
                    duplicate = False
                    for obj in self.vlanTags['objects']:
                        if obj['name'] == new_vlan['name']:
                            duplicate = True
                            break
                    if not duplicate:
                        self.vlanTags['objects'].append(new_vlan)
                        logging.info(f'Adding "{name}" to vlanTags for this AccessRules.')
                else:
                    self.vlanTags = {'objects': [{'name': vlantag.name, 'id': vlantag.id, 'type': vlantag.type}]}
                    logging.info(f'Adding "{name}" to vlanTags for this AccessRules.')
            else:
                logging.warning(f'VLAN Tag, "{name}", not found.  Cannot add to AccessRules.')
        elif action == 'remove':
            vlantag = VlanTags(fmc=self.fmc)
            vlantag.get(name=name)
            if 'id' in vlantag.__dict__:
                if 'vlanTags' in self.__dict__:
                    objects = []
                    for obj in self.vlanTags['objects']:
                        if obj['name'] != name:
                            objects.append(obj)
                    self.vlanTags['objects'] = objects
                    logging.info(f'Removed "{name}" from vlanTags for this AccessRules.')
                else:
                    logging.info("vlanTags doesn't exist for this AccessRules.  Nothing to remove.")
            else:
                logging.warning(f'VLAN Tag, {name}, not found.  Cannot remove from AccessRules.')
        elif action == 'clear':
            if 'vlanTags' in self.__dict__:
                del self.vlanTags
                logging.info('All VLAN Tags removed from this AccessRules object.')

    def source_zone(self, action, name=''):
        logging.debug("In source_zone() for AccessRules class.")
        if action == 'add':
            sz = SecurityZones(fmc=self.fmc)
            sz.get(name=name)
            if 'id' in sz.__dict__:
                if 'sourceZones' in self.__dict__:
                    new_zone = {'name': sz.name, 'id': sz.id, 'type': sz.type}
                    duplicate = False
                    for obj in self.sourceZones['objects']:
                        if obj['name'] == new_zone['name']:
                            duplicate = True
                            break
                    if not duplicate:
                        self.sourceZones['objects'].append(new_zone)
                        logging.info(f'Adding "{name}" to sourceZones for this AccessRules.')
                else:
                    self.sourceZones = {'objects': [{'name': sz.name, 'id': sz.id, 'type': sz.type}]}
                    logging.info(f'Adding "{name}" to sourceZones for this AccessRules.')
            else:
                logging.warning('Security Zone, "{name}", not found.  Cannot add to AccessRules.')
        elif action == 'remove':
            sz = SecurityZones(fmc=self.fmc)
            sz.get(name=name)
            if 'id' in sz.__dict__:
                if 'sourceZones' in self.__dict__:
                    objects = []
                    for obj in self.sourceZones['objects']:
                        if obj['name'] != name:
                            objects.append(obj)
                    self.sourceZones['objects'] = objects
                    logging.info(f'Removed "{name}" from sourceZones for this AccessRules.')
                else:
                    logging.info("sourceZones doesn't exist for this AccessRules.  Nothing to remove.")
            else:
                logging.warning(f'Security Zone, "{name}", not found.  Cannot remove from AccessRules.')
        elif action == 'clear':
            if 'sourceZones' in self.__dict__:
                del self.sourceZones
                logging.info('All Source Zones removed from this AccessRules object.')

    def destination_zone(self, action, name=''):
        logging.debug("In destination_zone() for AccessRules class.")
        if action == 'add':
            sz = SecurityZones(fmc=self.fmc)
            sz.get(name=name)
            if 'id' in sz.__dict__:
                if 'destinationZones' in self.__dict__:
                    new_zone = {'name': sz.name, 'id': sz.id, 'type': sz.type}
                    duplicate = False
                    for obj in self.destinationZones['objects']:
                        if obj['name'] == new_zone['name']:
                            duplicate = True
                            break
                    if not duplicate:
                        self.destinationZones['objects'].append(new_zone)
                        logging.info(f'Adding "{name}" to destinationZones for this AccessRules.')
                else:
                    self.destinationZones = {'objects': [{'name': sz.name, 'id': sz.id, 'type': sz.type}]}
                    logging.info(f'Adding "{name}" to destinationZones for this AccessRules.')
            else:
                logging.warning(f'Security Zone, "{name}", not found.  Cannot add to AccessRules.')
        elif action == 'remove':
            sz = SecurityZones(fmc=self.fmc)
            sz.get(name=name)
            if 'id' in sz.__dict__:
                if 'destinationZones' in self.__dict__:
                    objects = []
                    for obj in self.destinationZones['objects']:
                        if obj['name'] != name:
                            objects.append(obj)
                    self.destinationZones['objects'] = objects
                    logging.info('Removed "{name}" from destinationZones for this AccessRules.')
                else:
                    logging.info("destinationZones doesn't exist for this AccessRules.  Nothing to remove.")
            else:
                logging.warning(f'Security Zone, {name}, not found.  Cannot remove from AccessRules.')
        elif action == 'clear':
            if 'destinationZones' in self.__dict__:
                del self.destinationZones
                logging.info('All Destination Zones removed from this AccessRules object.')

    def source_port(self, action, name=''):
        logging.debug("In source_port() for AccessRules class.")
        if action == 'add':
            pport_json = ProtocolPortObjects(fmc=self.fmc)
            pport_json.get(name=name)
            if 'id' in pport_json.__dict__:
                item = pport_json
            else:
                item = PortObjectGroups(fmc=self.fmc)
                item.get(name=name)
            if 'id' in item.__dict__:
                if 'sourcePorts' in self.__dict__:
                    new_port = {'name': item.name, 'id': item.id, 'type': item.type}
                    duplicate = False
                    if 'objects' not in self.sourcePorts:
                        self.__dict__['sourcePorts']['objects'] = []
                    for obj in self.sourcePorts['objects']:
                        if obj['name'] == new_port['name']:
                            duplicate = True
                            break
                    if not duplicate:
                        self.sourcePorts['objects'].append(new_port)
                        logging.info(f'Adding "{name}" to sourcePorts for this AccessRules.')
                else:
                    self.sourcePorts = {'objects': [{'name': item.name, 'id': item.id, 'type': item.type}]}
                    logging.info(f'Adding "{name}" to sourcePorts for this AccessRules.')
            else:
                logging.warning(f'Protocol Port or Protocol Port Group: "{name}", '
                                f'not found.  Cannot add to AccessRules.')
        elif action == 'remove':
            pport_json = ProtocolPortObjects(fmc=self.fmc)
            pport_json.get(name=name)
            if 'id' in pport_json.__dict__:
                item = pport_json
            else:
                item = PortObjectGroups(fmc=self.fmc)
                item.get(name=name)
            if 'id' in item.__dict__:
                if 'sourcePorts' in self.__dict__:
                    objects = []
                    for obj in self.sourcePorts['objects']:
                        if obj['name'] != name:
                            objects.append(obj)
                    self.sourcePorts['objects'] = objects
                    logging.info(f'Removed "{name}" from sourcePorts for this AccessRules.')
                else:
                    logging.info("sourcePorts doesn't exist for this AccessRules.  Nothing to remove.")
            else:
                logging.warning(f'Protocol Port or Protocol Port Group: "{name}", '
                                f'not found.  Cannot add to AccessRules.')
        elif action == 'clear':
            if 'sourcePorts' in self.__dict__:
                del self.sourcePorts
                logging.info('All Source Ports removed from this AccessRules object.')

    def destination_port(self, action, name=''):
        logging.debug("In destination_port() for AccessRules class.")
        if action == 'add':
            pport_json = ProtocolPortObjects(fmc=self.fmc)
            pport_json.get(name=name)
            if 'id' in pport_json.__dict__:
                item = pport_json
            else:
                item = PortObjectGroups(fmc=self.fmc)
                item.get(name=name)
            if 'id' in item.__dict__:
                if 'destinationPorts' in self.__dict__:
                    new_port = {'name': item.name, 'id': item.id, 'type': item.type}
                    duplicate = False
                    if 'objects' not in self.destinationPorts:
                        self.__dict__['destinationPorts']['objects'] = []
                    for obj in self.destinationPorts['objects']:
                        if obj['name'] == new_port['name']:
                            duplicate = True
                            break
                    if not duplicate:
                        self.destinationPorts['objects'].append(new_port)
                        logging.info(f'Adding "{name}" to destinationPorts for this AccessRules.')
                else:
                    self.destinationPorts = {'objects': [{'name': item.name, 'id': item.id, 'type': item.type}]}
                    logging.info(f'Adding "{name}" to destinationPorts for this AccessRules.')
            else:
                logging.warning(f'Protocol Port or Protocol Port Group: "{name}", '
                                f'not found.  Cannot add to AccessRules.')
        elif action == 'remove':
            pport_json = ProtocolPortObjects(fmc=self.fmc)
            pport_json.get(name=name)
            if 'id' in pport_json.__dict__:
                item = pport_json
            else:
                item = PortObjectGroups(fmc=self.fmc)
                item.get(name=name)
            if 'id' in item.__dict__:
                if 'destinationPorts' in self.__dict__:
                    objects = []
                    for obj in self.destinationPorts['objects']:
                        if obj['name'] != name:
                            objects.append(obj)
                    self.destinationPorts['objects'] = objects
                    logging.info(f'Removed "{name}" from destinationPorts for this AccessRules.')
                else:
                    logging.info("destinationPorts doesn't exist for this AccessRules.  Nothing to remove.")
            else:
                logging.warning(f'Protocol Port or Protocol Port Group: "{name}", '
                                f'not found.  Cannot add to AccessRules.')
        elif action == 'clear':
            if 'destinationPorts' in self.__dict__:
                del self.destinationPorts
                logging.info('All Destination Ports removed from this AccessRules object.')

    def source_network(self, action, name='', literal=None):
        """
        Adds Either object having name=name or literal with {value:<>, type:<>} to the sourceNetworks
        field of AccessRules object
        Args:
            action: the action to be done
            name: name of the object in question
            literal: the literal in question
        Returns:
            None
        """
        # using dict() as default value is dangerous here, any thoughts/workarounds on this?
        logging.debug("In source_network() for AccessRules class.")
        if literal and name != '':
            raise ValueError('Only one of literals or name (object name) should be set while creating a source network')

        if not hasattr(self, 'sourceNetworks'):
            self.sourceNetworks = {'objects': [], 'literals': {}}

        if action == 'add':
            if literal:
                type_ = get_networkaddress_type(literal)
                self.sourceNetworks['literals'][literal] = type_
                logging.info(f'Adding literal "{literal}" of type "{type_}" to sourceNetworks for this AccessRules.')
            else:
                ipaddresses_json = NetworkAddresses(fmc=self.fmc).get()
                networkgroup_json = NetworkGroups(fmc=self.fmc).get()
                fqdns_json = FQDNS(fmc=self.fmc).get()
                items = ipaddresses_json.get('items', []) + \
                        networkgroup_json.get('items', []) + \
                        fqdns_json.get('items', [])
                new_net = None
                for item in items:
                    if item['name'] == name:
                        new_net = {'name': item['name'], 'id': item['id'], 'type': item['type']}
                        break
                if new_net is None:
                    logging.warning(f'Network "{name}" is not found in FMC.  Cannot add to sourceNetworks.')
                else:
                    if 'sourceNetworks' in self.__dict__:
                        # thus either some objects are already present in sourceNetworks,
                        # or only literals are present in sourceNetworks
                        if 'objects' in self.__dict__['sourceNetworks']:
                            # some objects are already present
                            duplicate = False
                            # see if its a duplicate or not. If not, append to the list of
                            # existing objects in sourceNetworks
                            for obj in self.sourceNetworks['objects']:
                                if obj['name'] == new_net['name']:
                                    duplicate = True
                                    break
                            if not duplicate:
                                self.sourceNetworks['objects'].append(new_net)
                                logging.info(f'Adding "{name}" to sourceNetworks for this AccessRules.')
                        else:
                            # this means no objects were present in sourceNetworks,
                            # and sourceNetworks contains literals only
                            self.sourceNetworks.update({'objects': [new_net]})
                            # So update the sourceNetworks dict which contained 'literals' key initially
                            # to have a 'objects' key as well
                            logging.info(f'Adding "{name}" to sourceNetworks for this AccessRules.')
                    else:
                        # None of literals or objects are present in sourceNetworks,
                        # so initialize it with objects and update the provided object
                        self.sourceNetworks = {'objects': [new_net]}
                        logging.info(f'Adding "{name}" to sourceNetworks for this AccessRules.')
        elif action == 'remove':
            if 'sourceNetworks' in self.__dict__:
                if name != '':
                    # an object's name has been provided to be removed
                    objects = []
                    for obj in self.sourceNetworks['objects']:
                        if obj['name'] != name:
                            objects.append(obj)
                    if len(objects) == 0:
                        # it was the last object which was deleted now
                        del self.sourceNetworks
                        logging.info(f'Removed "{name}" from sourceNetworks for this AccessRules')
                        logging.info('All Source Networks removed from this AccessRules object.')
                    else:
                        self.sourceNetworks['objects'] = objects
                        logging.info(f'Removed "{name}" from sourceNetworks for this AccessRules.')
                else:
                    # a literal value has been provided to be removed
                    type_ = self.sourceNetworks['literals'].get(literal)
                    if type_:
                        self.sourceNetworks['literals'].pop(literal)
                        logging.info(f'Removed literal "{literal}" of type '
                                     f'"{type_}" from sourceNetworks for this AccessRules.')
                    else:
                        logging.info(f'Unable to removed literal "{literal}" from sourceNetworks as it was not found')
            else:
                logging.info("sourceNetworks doesn't exist for this AccessRules.  Nothing to remove.")
        elif action == 'clear':
            if 'sourceNetworks' in self.__dict__:
                del self.sourceNetworks
                logging.info('All Source Networks removed from this AccessRules object.')

    def destination_network(self, action, name='', literal=None):
        """
        Adds Either object having name=name or literal with {value:<>, type:<>} to the sourceNetworks
        field of AccessRules object
        Args:
            action: the action to be done
            name: name of the object in question
            literal: the literal in question
        Returns:
            None
        """
        # using dict() as default value is dangerous here, any thoughts/workarounds on this?

        logging.debug("In destination_network() for ACPRule class.")
        if literal and name != '':
            raise ValueError('Only one of literals or name (object name) should be set while creating a source network')

        if not hasattr(self, 'destinationNetworks'):
            self.destinationNetworks = {'objects': [], 'literals': {}}

        if action == 'add':
            if literal:
                type_ = get_networkaddress_type(literal)
                self.destinationNetworks['literals'][literal] = type_
                logging.info(f'Adding literal "{literal}" of type "{type_}" '
                             f'to destinationNetworks for this AccessRules.')
            else:
                ipaddresses_json = NetworkAddresses(fmc=self.fmc).get()
                networkgroup_json = NetworkGroups(fmc=self.fmc).get()
                if self.fmc.serverVersion >= '6.4':
                    fqdns_json = FQDNS(fmc=self.fmc).get()
                else:
                    fqdns_json = {'items': []}
                items = ipaddresses_json.get('items', []) + \
                    networkgroup_json.get('items', []) + \
                    fqdns_json.get('items', [])
                new_net = None
                for item in items:
                    if item['name'] == name:
                        new_net = {'name': item['name'], 'id': item['id'], 'type': item['type']}
                        break
                if new_net is None:
                    logging.warning(f'Network "{name}" is not found in FMC.  Cannot add to destinationNetworks.')
                else:
                    if 'destinationNetworks' in self.__dict__:
                        # thus either some objects are already present in destinationNetworks,
                        # or only literals are present in destinationNetworks
                        if 'objects' in self.__dict__['destinationNetworks']:
                            # some objects are already present
                            duplicate = False
                            for obj in self.destinationNetworks['objects']:
                                if obj['name'] == new_net['name']:
                                    duplicate = True
                                    break
                            if not duplicate:
                                self.destinationNetworks['objects'].append(new_net)
                                logging.info(f'Adding "{name}" to destinationNetworks for this AccessRules.')
                        else:
                            # this means no objects were present in destinationNetworks,
                            # and destinationNetworks contains literals only
                            self.destinationNetworks.update({'objects': [new_net]})
                            # So update the destinationNetworks dict which contained 'literals' key initially
                            # to have a 'objects' key as well
                            logging.info(f'Adding "{name}" to destinationNetworks for this AccessRules.')
                    else:
                        # None of literals or objects are present in destinationNetworks,
                        # so initialize it with objects and update the provided object
                        self.destinationNetworks = {'objects': [new_net]}
                        logging.info(f'Adding "{name}" to destinationNetworks for this AccessRules.')
        elif action == 'remove':
            if 'destinationNetworks' in self.__dict__:
                if name != '':
                    # an object's name has been provided to be removed
                    objects = []
                    for obj in self.destinationNetworks['objects']:
                        if obj['name'] != name:
                            objects.append(obj)
                    if len(objects) == 0:
                        # it was the last object which was deleted now
                        del self.destinationNetworks
                        logging.info(f'Removed "{name}" from destinationNetworks for this AccessRules')
                        logging.info('All Destination Networks removed from this AccessRules object.')
                    else:
                        self.destinationNetworks['objects'] = objects
                        logging.info(f'Removed "{name}" from destinationNetworks for this AccessRules.')
                else:
                    # a literal value has been provided to be removed
                    type_ = self.destinationNetworks['literals'].get(literal)
                    if type_:
                        self.destinationNetworks['literals'].pop(literal)
                        logging.info(f'Removed literal "{literal}" of '
                                     f'type "{type_}" from destinationNetworks for this AccessRules.')
                    else:
                        logging.info(f'Unable to removed literal "{literal}" '
                                     f'from destinationNetworks as it was not found')
            else:
                logging.info("destinationNetworks doesn't exist for this AccessRules.  Nothing to remove.")
        elif action == 'clear':
            if 'destinationNetworks' in self.__dict__:
                del self.destinationNetworks
                logging.info('All Destination Networks removed from this AccessRules object.')

    def source_sgt(self, action, name='', literal=None):
        """
        Adds Either object having name=name or literal with {value:<>, type:<>} to the sourceSecurityGroupTags
        field of AccessRules object
        Args:
            action: the action to be done
            name: name of the object in question
            literal: the literal in question
        Returns:
            None
        """
        # using dict() as default value is dangerous here, any thoughts/workarounds on this?

        logging.debug("In destination_network() for ACPRule class.")
        if literal and name != '':
            raise ValueError('Only one of literals or name (object name) should be set while creating a source network')

        if not hasattr(self, 'sourceSecurityGroupTags'):
            self.sourceSecurityGroupTags = {'objects': [], 'literals': {}}

        if action == 'add':
            if literal:
                type_ = 'sourceSecurityGroupTags'  # This is probably wrong.
                self.sourceSecurityGroupTags['literals'][literal] = type_
                logging.info(f'Adding literal "{literal}" of type "{type_}" '
                             f'to sourceSecurityGroupTags for this AccessRules.')
            else:
                # Query FMC for all SGTs and iterate through them to see if our name matches 1 of them.
                sgt = ISESecurityGroupTags(fmc=self.fmc)
                sgt.get(name=name)
                if 'id' in sgt.__dict__:
                    item = sgt
                else:
                    item = {}
                new_sgt = None
                if item['name'] == name:
                    new_sgt = {'name': item['name'], 'tag': item['tag'], 'type': item['type']}
                if new_sgt is None:
                    logging.warning(f'SecurityGroupTag "{name}" is not found in FMC.  '
                                    f'Cannot add to sourceSecurityGroupTags.')
                else:
                    if 'sourceSecurityGroupTags' in self.__dict__:
                        # thus either some objects are already present in sourceSecurityGroupTags,
                        # or only literals are present in sourceSecurityGroupTags
                        if 'objects' in self.__dict__['sourceSecurityGroupTags']:
                            # some objects are already present
                            duplicate = False
                            for obj in self.sourceSecurityGroupTags['objects']:
                                if obj['name'] == new_sgt['name']:
                                    duplicate = True
                                    break
                            if not duplicate:
                                self.sourceSecurityGroupTags['objects'].append(new_sgt)
                                logging.info(f'Adding "{name}" to sourceSecurityGroupTags for this AccessRules.')
                        else:
                            # this means no objects were present in sourceSecurityGroupTags,
                            # and sourceSecurityGroupTags contains literals only
                            self.sourceSecurityGroupTags.update({'objects': [new_sgt]})
                            # So update the sourceSecurityGroupTags dict which contained 'literals' key initially
                            # to have a 'objects' key as well
                            logging.info(f'Adding "{name}" to sourceSecurityGroupTags for this AccessRules.')
                    else:
                        # None of literals or objects are present in sourceSecurityGroupTags,
                        # so initialize it with objects and update the provided object
                        self.sourceSecurityGroupTags = {'objects': [new_sgt]}
                        logging.info(f'Adding "{name}" to sourceSecurityGroupTags for this AccessRules.')
        elif action == 'remove':
            if 'sourceSecurityGroupTags' in self.__dict__:
                if name != '':
                    # an object's name has been provided to be removed
                    objects = []
                    for obj in self.sourceSecurityGroupTags['objects']:
                        if obj['name'] != name:
                            objects.append(obj)
                    if len(objects) == 0:
                        # it was the last object which was deleted now
                        del self.sourceSecurityGroupTags
                        logging.info(f'Removed "{name}" from sourceSecurityGroupTags for this AccessRules')
                        logging.info('All source security group tags are removed from this AccessRules object.')
                    else:
                        self.sourceSecurityGroupTags['objects'] = objects
                        logging.info(f'Removed "{name}" from sourceSecurityGroupTags for this AccessRules.')
                else:
                    # a literal value has been provided to be removed
                    type_ = self.sourceSecurityGroupTags['literals'].get(literal)
                    if type_:
                        self.sourceSecurityGroupTags['literals'].pop(literal)
                        logging.info(f'Removed literal "{literal}" of '
                                     f'type "{type_}" from sourceSecurityGroupTags for this AccessRules.')
                    else:
                        logging.info(f'Unable to removed literal "{literal}" '
                                     f'from sourceSecurityGroupTags as it was not found')
            else:
                logging.info("No sourceSecurityGroupTags exist for this AccessRules.  Nothing to remove.")
        elif action == 'clear':
            if 'sourceSecurityGroupTags' in self.__dict__:
                del self.sourceSecurityGroupTags
                logging.info('All source security group tags are removed from this AccessRules object.')

    def destination_sgt(self, action, name='', literal=None):
        pass


class ACPRule(AccessRules):
    """Dispose of this Class after 20210101."""

    def __init__(self, fmc, **kwargs):
        warnings.resetwarnings()
        warnings.warn("Deprecated: ACPRule() should be called via AccessRules().")
        super().__init__(fmc, **kwargs)


class Bulk(object):
    """Bulk Class"""
    MAX_SIZE_QTY = 1000
    MAX_SIZE_IN_BYTES = 2048000
    REQUIRED_FOR_POST = []

    @property
    def URL_SUFFIX(self):
        """
        Add the URL suffixes for section, categories, insertBefore and insertAfter.
        """
        url = '?'

        if 'category' in self.__dict__:
            url = f'{url}category={self.category}&'
        if 'insertBefore' in self.__dict__:
            url = f'{url}insertBefore={self.insertBefore}&'
        if 'insertAfter' in self.__dict__:
            url = f'{url}insertAfter={self.insertAfter}&'
        if 'insertBefore' in self.__dict__ and 'insertAfter' in self.__dict__:
            logging.warning('ACP rule has both insertBefore and insertAfter params')
        if 'section' in self.__dict__:
            url = f'{url}section={self.section}&'

        return url[:-1]

    def __init__(self, fmc, url='', **kwargs):
        logging.debug("In __init__() for Bulk class.")
        self.fmc = fmc
        self.items = []
        self.URL = url
        self.parse_kwargs(**kwargs)

    def parse_kwargs(self, **kwargs):
        logging.debug("In parse_kwargs() for Bulk class.")
        if 'category' in kwargs:
            self.category = kwargs['category']
        if 'insertBefore' in kwargs:
            self.insertBefore = kwargs['insertBefore']
        if 'insertAfter' in kwargs:
            self.insertAfter = kwargs['insertAfter']
        if 'section' in kwargs:
            self.section = kwargs['section']

    def add(self, item):
        self.items.append(item)
        logging.info(f"Adding {item} to bulk items list.")

    def clear(self):
        logging.info(f"Clearing bulk items list.")
        self.items = []

    def post(self):
        # Build URL
        self.URL = f'{self.URL}{self.URL_SUFFIX}&bulk=true'

        # Break up the items into MAX_BULK_POST_SIZE chunks.
        chunks = [self.items[i * self.MAX_SIZE_QTY:(i + 1) * self.MAX_SIZE_QTY]
                  for i in range((len(self.items) + self.MAX_SIZE_QTY - 1) // self.MAX_SIZE_QTY)]

        # Post the chunks
        for item in chunks:
            # I'm not sure what to do about the max bytes right now so I'll just throw a warning message.
            if sys.getsizeof(item, 0) > self.MAX_SIZE_IN_BYTES:
                logging.warning(f"This chunk of the post is too large.  Please submit less items to be bulk posted.")
            response = self.fmc.send_to_api(method='post', url=self.URL, json_data=item)
            logging.info(f"Posting to bulk items.")
            return response



import logging
import fmcapi
import time


def test__port_object_group(fmc):
    logging.info('Testing PortObjectGroup class.')

    starttime = str(int(time.time()))
    namer = f'_fmcapi_test_{starttime}'

    obj10 = fmcapi.ProtocolPortObjects(fmc=fmc, name='_porttcp1', port='8443', protocol='TCP')
    obj10.post()
    obj11 = fmcapi.ProtocolPortObjects(fmc=fmc, name='_portudp1', port='161', protocol='UDP')
    obj11.post()
    obj12 = fmcapi.ProtocolPortObjects(fmc=fmc, name='_portrangetcp1', port='0-1023', protocol='TCP')
    obj12.post()
    time.sleep(1)
    obj1 = fmcapi.PortObjectGroups(fmc=fmc, name=namer)
    obj1.named_ports(action='add', name=obj10.name)
    obj1.named_ports(action='add', name=obj10.name)
    obj1.named_ports(action='remove', name=obj10.name)
    obj1.named_ports(action='clear')
    obj1.named_ports(action='add', name=obj11.name)
    obj1.named_ports(action='add', name=obj12.name)
    obj1.named_ports(action='remove', name=obj11.name)
    obj1.post()
    time.sleep(1)
    del obj1

    obj1 = fmcapi.PortObjectGroups(fmc=fmc, name=namer)
    obj1.get()
    obj1.named_ports(action='add', name='HTTP')
    obj1.named_ports(action='clear')
    obj1.named_ports(action='add', name='HTTP')
    obj1.named_ports(action='remove', name='HTTP')
    obj1.named_ports(action='add', name='HTTP')
    obj1.named_ports(action='add', name='HTTPS')
    obj1.put()

    time.sleep(1)
    obj1.delete()
    obj10.delete()
    obj11.delete()
    obj12.delete()

    logging.info('Testing PortObjectGroup class done.\n')



import logging
import fmcapi
import time


def test__ip_range(fmc):
    logging.info('Test IPRange.  Post, get, put, delete Range Objects.')

    starttime = str(int(time.time()))
    namer = f'_fmcapi_test_{starttime}'

    obj1 = fmcapi.Ranges(fmc=fmc)
    obj1.name = namer
    obj1.value = '1.1.1.1-2.2.2.2'
    obj1.post()
    time.sleep(1)
    del obj1

    obj1 = fmcapi.Ranges(fmc=fmc, name=namer)
    obj1.get()
    obj1.value = '3.3.3.3-4.4.4.4'
    obj1.put()
    time.sleep(1)
    obj1.delete()

    logging.info('Test IPRange done.\n')



import logging
import fmcapi


def test__geolocations(fmc):
    logging.info('Testing Geolocation class. Requires a configured Geolocation')

    obj1 = fmcapi.Geolocation(fmc=fmc)
    logging.info('All Geolocation -- >')
    result = obj1.get()
    logging.info(result)
    logging.info(f"Total items: {len(result['items'])}")
    del obj1

    obj1 = fmcapi.Geolocation(fmc=fmc, name='_tmp')
    logging.info('One Geolocation -- >')
    logging.info(obj1.get())

    logging.info('Testing Geolocation class done.\n')



import logging
from .anyprotocolportobjects import AnyProtocolPortObjects
from .applications import Applications
from .applications import Application
from .applicationcategories import ApplicationCategories
from .applicationcategories import ApplicationCategory
from .applicationfilters import ApplicationFilters
from .applicationfilters import ApplicationFilter
from .applicationproductivities import ApplicationProductivities
from .applicationproductivities import ApplicationProductivity
from .applicationrisks import ApplicationRisks
from .applicationrisks import ApplicationRisk
from .applicationtags import ApplicationTags
from .applicationtags import ApplicationTag
from .applicationtypes import ApplicationTypes
from .applicationtypes import ApplicationType
from .certenrollments import CertEnrollments
from .certenrollments import CertEnrollment
from .continents import Continents
from .continents import Continent
from .countries import Countries
from .countries import Country
from .dnsservergroups import DNSServerGroups
from .endpointdevicetypes import EndPointDeviceTypes
from .extendedaccesslist import ExtendedAccessList
from .fqdns import FQDNS
from .geolocation import Geolocation
from .icmpv4objects import ICMPv4Objects
from .icmpv4objects import ICMPv4Object
from .icmpv6objects import ICMPv6Objects
from .icmpv6objects import ICMPv6Object
from .ikev1ipsecproposals import IKEv1IpsecProposals
from .ikev1policies import IKEv1Policies
from .ikev2ipsecproposals import IKEv2IpsecProposals
from .ikev2policies import IKEv2Policies
from .interfacegroups import InterfaceGroups
from .interfacegroups import InterfaceGroup
from .interfaceobjects import InterfaceObjects
from .interfaceobjects import InterfaceObject
from .networkaddresses import NetworkAddresses
from .networkaddresses import IPAddresses
from .hosts import Hosts
from .hosts import IPHost
from .networks import Networks
from .networks import IPNetwork
from .ranges import Ranges
from .ranges import IPRange
from .isesecuritygrouptags import ISESecurityGroupTags
from .networkgroups import NetworkGroups
from .networkgroups import NetworkGroup
from .portobjectgroups import PortObjectGroups
from .portobjectgroups import PortObjectGroup
from .ports import Ports
from .protocolportobjects import ProtocolPortObjects
from .protocolportobjects import ProtocolPort
from .realms import Realms
from .realmusergroups import RealmUserGroups
from .realmusers import RealmUsers
from .securitygrouptags import SecurityGroupTags
from .securityzones import SecurityZones
from .securityzones import SecurityZone
from .siurlfeeds import SIUrlFeeds
from .siurllists import SIUrlLists
from .slamonitors import SLAMonitors
from .slamonitors import SLAMonitor
from .tunneltags import TunnelTags
from .urls import URLs
from .urls import URL
from .urlcategories import URLCategories
from .urlcategories import URLCategory
from .urlgroups import URLGroups
from .urlgroups import URLGroup
from .variablesets import VariableSets
from .variablesets import VariableSet
from .vlangrouptags import VlanGroupTags
from .vlangrouptags import VlanGroupTag
from .vlantags import VlanTags
from .vlantags import VlanTag

logging.debug("In the object_services __init__.py file.")

__all__ = [
    'AnyProtocolPortObjects',
    'ApplicationCategories',
    'ApplicationCategory',
    'Applications',
    'Application',
    'ApplicationFilters',
    'ApplicationFilter',
    'ApplicationProductivities',
    'ApplicationProductivity',
    'ApplicationRisks',
    'ApplicationRisk',
    'ApplicationTags',
    'ApplicationTag',
    'ApplicationTypes',
    'ApplicationType',
    'CertEnrollments',
    'CertEnrollment',
    'Continents',
    'Continent',
    'Countries',
    'Country',
    'DNSServerGroups',
    'EndPointDeviceTypes',
    'ExtendedAccessList',
    'FQDNS',
    'Geolocation',
    'Hosts',
    'IPHost',
    'ICMPv4Objects',
    'ICMPv4Object',
    'ICMPv6Objects',
    'ICMPv6Object',
    'IKEv1IpsecProposals',
    'IKEv1Policies',
    'IKEv2IpsecProposals',
    'IKEv2Policies',
    'InterfaceGroups',
    'InterfaceGroup',
    'InterfaceObjects',
    'InterfaceObject',
    'ISESecurityGroupTags',
    'NetworkAddresses',
    'IPAddresses',
    'NetworkGroups',
    'NetworkGroup',
    'Networks',
    'IPNetwork',
    'PortObjectGroups',
    'PortObjectGroup',
    'Ports',
    'ProtocolPortObjects',
    'ProtocolPort',
    'Ranges',
    'IPRange',
    'Realms',
    'RealmUserGroups',
    'RealmUsers',
    'SecurityGroupTags',
    'SecurityZones',
    'SecurityZone',
    'SIUrlFeeds',
    'SIUrlLists',
    'SLAMonitors',
    'SLAMonitor',
    'TunnelTags',
    'URLCategories',
    'URLCategory',
    'URLGroups',
    'URLGroup',
    'URLs',
    'URL',
    'VariableSets',
    'VariableSet',
    'VlanGroupTags',
    'VlanGroupTag',
    'VlanTags',
    'VlanTag',
]



import logging
import fmcapi
import time


def test__security_zone(fmc):
    logging.info('Test SecurityZone.  Post, get, put, delete Security Zone Objects.')

    starttime = str(int(time.time()))
    namer = f'_fmcapi_test_{starttime}'

    obj1 = fmcapi.SecurityZones(fmc=fmc)
    obj1.name = namer
    obj1.interfaceMode = 'ROUTED'
    obj1.post()
    time.sleep(1)
    del obj1
    obj1 = fmcapi.SecurityZones(fmc=fmc, name=namer)
    obj1.get()
    obj1.name = 'DEMO'
    obj1.put()
    time.sleep(1)
    obj1.delete()
    logging.info('Test SecurityZone done.\n')



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
import logging
import warnings


class URLs(APIClassTemplate):
    """
    The URLs Object in the FMC.
    """

    VALID_JSON_DATA = ['id', 'name', 'url', 'description']
    VALID_FOR_KWARGS = VALID_JSON_DATA + []
    URL_SUFFIX = '/object/urls'
    REQUIRED_FOR_POST = ['name', 'url']

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for URLs class.")
        self.parse_kwargs(**kwargs)


class URL(URLs):
    """Dispose of this Class after 20210101."""

    def __init__(self, fmc, **kwargs):
        warnings.resetwarnings()
        warnings.warn("Deprecated: URL() should be called via URLs().")
        super().__init__(fmc, **kwargs)



import logging
import fmcapi


def test__filepolicies(fmc):
    logging.info('Testing FilePolicies class.')

    obj1 = fmcapi.FilePolicies(fmc=fmc)
    logging.info('All FilePolicies -- >')
    result = obj1.get()
    logging.info(result)
    logging.info(f"Total items: {len(result['items'])}")

    del obj1

    logging.info('Testing FilePolicies class done.\n')



import fmcapi
import time


def wait_for_task(fmc, task, wait_time=10):
    task_completed_states = ['Success', 'SUCCESS', 'COMPLETED']
    try:
        status = fmcapi.TaskStatuses(fmc=fmc, id=task["id"])
        current_status = status.get()
        '''
        Task Status for new device registration behaves differently than other tasks
        On new device registration, a task is sent for the initial registration. After completion 
        the UUID is deleted without any change in task status. So we check to see if the object no longer exists
        to assume the registration is complete.  After registration, discovery of the device begins, but there is
        no way to check for this with a task status.  The device can't be modified during this time, but a new device
        registration can begin.

        OTOH, a device HA operation will update its status to "Success" on completion.  Hence the two different checks.
        '''
        while current_status["status"] is not None and current_status["status"] not in task_completed_states:
            # Lot of inconsistencies with the type of data a task can return
            if 'taskType' in current_status.keys():
                logging.info("Task: %s %s %s" % (
                    current_status["taskType"], current_status["status"], current_status["id"]))
                time.sleep(wait_time)
                current_status = status.get()
            else:
                logging.info("Task: %s %s" % (
                    current_status["status"], current_status["id"]))
                time.sleep(wait_time)
                current_status = status.get()
        logging.info("Task: %s %s" % (
            current_status["status"], current_status["id"]))
    except Exception as e:
        logging.info(type(e), e)



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
from fmcapi.api_objects.device_services.devicerecords import DeviceRecords
from .upgradepackages import UpgradePackages
import logging


class Upgrades(APIClassTemplate):
    """
    Change this class to UpgradePackage once the deprecated UpgradePackage name for UpgradePackages expires in 2021.
    The Upgrades Object in the FMC.
    NOTE:  This should be called UpgradePackage but that collides with a Deprecated name for UpgradePackages.
    We can rename this after we remove that deprecation... which will be a while from now.
    """

    VALID_JSON_DATA = ['id', 'name', 'type', 'upgradePackage', 'targets', 'pushUpgradeFileOnly']
    VALID_FOR_KWARGS = VALID_JSON_DATA + []
    URL_SUFFIX = '/updates/upgrades'

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for Upgrades class.")
        self.type = 'Upgrade'
        self.URL = f'{self.fmc.platform_url}{self.URL_SUFFIX}'
        self.parse_kwargs(**kwargs)

    def upgrade_package(self, package_name):
        logging.debug("In upgrade_package() for Upgrades class.")
        package1 = UpgradePackages(fmc=self.fmc)
        package1.get(name=package_name)
        if 'id' in package1.__dict__:
            self.upgradePackage = {"id": package1.id, "type": package1.type}
        else:
            logging.warning(f'UpgradePackage "{package_name}" not found.  Cannot add package to Upgrades.')

    def devices(self, devices):
        logging.debug("In devices() for Upgrades class.")
        for device in devices:
            device1 = DeviceRecords(fmc=self.fmc)
            device1.get(name=device)
            if 'id' in device1.__dict__ and 'targets' in self.__dict__:
                self.targets.append({"id": device1.id, "type": device1.type, "name": device1.name})
            elif 'id' in device1.__dict__:
                self.targets = [{"id": device1.id, "type": device1.type, "name": device1.name}]
            else:
                logging.warning(f'Device "{device}" not found.  Cannot prepare devices for Upgrades.')

    def get(self):
        logging.info('GET method for API for Upgrades not supported.')
        pass

    def post(self, **kwargs):
        # returns a task status object
        logging.debug("In post() for Upgrades class.")
        self.fmc.autodeploy = False
        return super().post(**kwargs)

    def put(self):
        logging.info('PUT method for API for Upgrades not supported.')
        pass

    def delete(self):
        logging.info('DELETE method for API for Upgrades not supported.')
        pass



import logging
import fmcapi


def test__fqdns(fmc):
    logging.info('Test FQDNS.  Post, get, put, delete FQDNS Objects.')

    starttime = str(int(time.time()))
    namer = f'_fmcapi_test_{starttime}'

    obj1 = fmcapi.FQDNS(fmc=fmc)
    obj1.name = "_fqdns1" + namer
    obj1.value = "www.cisco.com"
    obj1.dnsResolution = "IPV4_ONLY"
    obj1.post()

    obj1.get()
    obj1.dnsResolution = "IPV4_AND_IPV6"
    obj1.put()

    obj1.delete()

    logging.info('FQDNS DNSServerGroups class done.\n')



import logging
import fmcapi


def test__continent(fmc):
    logging.info('Testing Continent class.')

    obj1 = fmcapi.Continents(fmc=fmc)
    logging.info('All Continents -- >')
    result = obj1.get()
    logging.info(result)
    logging.info(f"Total items: {len(result['items'])}")
    logging.info('\n')
    del obj1
    obj1 = fmcapi.Continents(fmc=fmc, name='North America')
    logging.info('One Continent -- >')
    logging.info(obj1.get())
    logging.info('\n')

    logging.info('Testing Continent class done.\n')



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
from fmcapi.api_objects.helper_functions import *
import logging
import warnings


class Ranges(APIClassTemplate):
    """
    The Ranges Object in the FMC.
    """

    VALID_JSON_DATA = ['id', 'name', 'value', 'description']
    VALID_FOR_KWARGS = VALID_JSON_DATA + []
    URL_SUFFIX = '/object/ranges'
    REQUIRED_FOR_POST = ['name', 'value']

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for Ranges class.")
        self.parse_kwargs(**kwargs)
        if 'value' in kwargs:
            value_type = get_networkaddress_type(kwargs['value'])
            if value_type == 'host' or value_type == 'network':
                logging.warning(f"value, {kwargs['value']}, is of type {value_type}.  Limited functionality for this "
                                f"object due to it being created via the Ranges function.")
            if validate_ip_bitmask_range(value=kwargs['value'], value_type=value_type):
                self.value = kwargs['value']
            else:
                logging.error(f"Provided value, {kwargs['value']}, has an error with the IP address(es).")


class IPRange(Ranges):
    """Dispose of this Class after 20210101."""

    def __init__(self, fmc, **kwargs):
        warnings.resetwarnings()
        warnings.warn("Deprecated: IPRange() should be called via Ranges().")
        super().__init__(fmc, **kwargs)



import logging
import fmcapi


def test__ip_addresses(fmc):
    logging.info('Test IPAddresses.  This only returns a full list of IP object types.')

    obj1 = fmcapi.NetworkAddresses(fmc=fmc)
    logging.info('IPAddresses -->')
    result = obj1.get()
    logging.info(result)

    logging.info('Test IPAddresses done.\n')



import logging
import fmcapi


def test__url_category(fmc):
    logging.info('Testing URLCategory class.')
    obj1 = fmcapi.URLCategories(fmc=fmc)
    logging.info('All URLCategories -- >')
    result = obj1.get()
    logging.info(result)
    logging.info(f"Total items: {len(result['items'])}")

    del obj1
    obj1 = fmcapi.URLCategories(fmc=fmc, name='SPAM URLs')
    logging.info('One URLCategory -- >')
    logging.info(obj1.get())

    logging.info('Testing URLCategory class done.\n')



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
import logging
import warnings


class ICMPv6Objects(APIClassTemplate):
    """
    The ICMPv6Objects Object in the FMC.
    """

    VALID_JSON_DATA = ['id', 'name', 'type', 'overrideTargetId', 'code', 'icmpType', 'overrides', 'overridable']
    VALID_FOR_KWARGS = VALID_JSON_DATA + []
    URL_SUFFIX = '/object/icmpv6objects'
    VALID_CHARACTERS_FOR_NAME = """[.\w\d_\- ]"""

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for ICMPv6Objects class.")
        self.parse_kwargs(**kwargs)
        self.type = 'ICMPV6Object'


class ICMPv6Object(ICMPv6Objects):
    """Dispose of this Class after 20210101."""

    def __init__(self, fmc, **kwargs):
        warnings.resetwarnings()
        warnings.warn("Deprecated: ICMPv6Object() should be called via ICMPv6Objects().")
        super().__init__(fmc, **kwargs)



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
from .ftddevicehapairs import FTDDeviceHAPairs
import logging
import warnings


class MonitoredInterfaces(APIClassTemplate):
    """
    The MonitoredInterfaces Object in the FMC.
    """

    VALID_JSON_DATA = ['id', 'name', 'ipv4Configuration', 'ipv6Configuration', 'monitorForFailures']
    VALID_FOR_KWARGS = VALID_JSON_DATA + ['ha_name']
    PREFIX_URL = '/devicehapairs/ftddevicehapairs'
    REQUIRED_FOR_PUT = ['id']

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for MonitoredInterfaces class.")
        self.parse_kwargs(**kwargs)

    def parse_kwargs(self, **kwargs):
        super().parse_kwargs(**kwargs)
        logging.debug("In parse_kwargs() for MonitoredInterfaces class.")
        if 'ha_name' in kwargs:
            self.device_ha(ha_name=kwargs['ha_name'])

    def device_ha(self, ha_name):
        logging.debug("In device_ha() for MonitoredInterfaces class.")
        deviceha1 = FTDDeviceHAPairs(fmc=self.fmc, name=ha_name)
        deviceha1.get()
        if 'id' in deviceha1.__dict__:
            self.deviceha_id = deviceha1.id
            self.URL = f'{self.fmc.configuration_url}{self.PREFIX_URL}/{self.deviceha_id}/monitoredinterfaces'
            self.deviceha_added_to_url = True
        else:
            logging.warning(f'Device HA {ha_name} not found.  Cannot set up device for MonitoredInterfaces.')

    def ipv4(self, ipv4addr, ipv4mask, ipv4standbyaddr):
        logging.debug("In ipv4() for MonitoredInterfaces class.")
        self.ipv4Configuration = {
            "activeIPv4Address": ipv4addr,
            "activeIPv4Mask": ipv4mask,
            "standbyIPv4Address": ipv4standbyaddr}

    def post(self):
        logging.info('POST method for API for MonitoredInterfaces not supported.')
        pass


class DeviceHAMonitoredInterfaces(MonitoredInterfaces):
    """Dispose of this Class after 20210101."""

    def __init__(self, fmc, **kwargs):
        warnings.resetwarnings()
        warnings.warn("Deprecated: DeviceHAMonitoredInterfaces() should be called via MonitoredInterfaces().")
        super().__init__(fmc, **kwargs)



import logging
import fmcapi
import time


def test__ipv6staticroutes(fmc):
    logging.info(
        'Testing IPv6StaticRoutes class. get, post, put, delete IPv4StaticRoute Objects. Requires a registered device')

    starttime = str(int(time.time()))
    namer = f'_fmcapi_test_{starttime}'

    iphost1 = fmcapi.Hosts(fmc=fmc, name='_iphost1' + namer, value='2001:db8::1')
    iphost1.post()
    ipnet1 = fmcapi.Networks(fmc=fmc, name='_ipnet1' + namer, value='2001:db8:1::/64')
    ipnet2 = fmcapi.Networks(fmc=fmc, name='_ipnet2' + namer, value='2001:db8:2::/64')
    ipnet1.post()
    ipnet2.post()

    ipv4route1 = fmcapi.IPv6StaticRoutes(fmc=fmc, name='_ipv6route1')
    ipv4route1.device(device_name='ftdv01.ccie.lab')
    ipv4route1.networks(action='add', networks=[ipnet1.name, ipnet2.name])
    ipv4route1.gw(name=iphost1.name)
    ipv4route1.interfaceName = "ifname"
    ipv4route1.metricValue = 1
    result = ipv4route1.post()

    ipv4route2 = fmcapi.IPv4StaticRoutes(fmc=fmc, name='_ipv6route1')
    ipv4route2.device(device_name='device_name')
    ipv4route2.id = result['id']
    ipv4route2.get()

    del ipv4route1
    ipv4route2.networks(action='remove', networks=[ipnet2.name])
    ipv4route2.put()

    ipv4route2.delete()
    ipnet1.delete()
    ipnet2.delete()
    iphost1.delete()
    logging.info('Testing IPv6StaticRoutes class done.\n')



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
import logging


class IKEv2IpsecProposals(APIClassTemplate):
    """
    The IKEv2IpsecProposals Object in the FMC.
    """

    VALID_JSON_DATA = ['id', 'name', 'type', 'encryptionAlgorithms', 'integrityAlgorithms']
    VALID_FOR_KWARGS = VALID_JSON_DATA + []
    URL_SUFFIX = '/object/ikev2ipsecproposals'
    REQUIRED_FOR_POST = ['name', 'encryptionAlgorithms', 'integrityAlgorithms']
    VALID_FOR_ENCRYPTION = ['DES', '3DES', 'AES', 'AES-192', 'AES-256', 'NULL', 'AES-GCM', 'AES-GCM-192',
                            'AES-GCM-256', 'AES-GMAC', 'AES-GMAC-192', 'AES-GMAC-256',
                            ]
    VALID_FOR_HASH = ['NULL', 'MD5', 'SHA-1', 'SHA-256', 'SHA-384', 'SHA-512']
    VALID_CHARACTERS_FOR_NAME = """[.\w\d_\- ]"""
    FIRST_SUPPORTED_FMC_VERSION = '6.3'

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for IKEv2IpsecProposals class.")
        self.parse_kwargs(**kwargs)
        self.type = 'IKEv2IPsecProposal'

    def encryption(self, action, algorithms=[]):
        logging.debug("In encryption() for IKEv2IpsecProposals class.")
        if action == 'add':
            for algorithm in algorithms:
                if 'encryptionAlgorithms' in self.__dict__:
                    if algorithm in self.encryptionAlgorithms:
                        logging.warning(f'encryptionAlgorithms "{algorithm}" already exists.')
                    elif algorithm in self.VALID_FOR_ENCRYPTION:
                        self.encryptionAlgorithms.append(algorithm)
                    else:
                        logging.warning(f'encryptionAlgorithms "{algorithm}" not a valid type.')
                else:
                    self.encryptionAlgorithms = [algorithm]
        elif action == 'remove':
            if 'encryptionAlgorithms' in self.__dict__:
                for algorithm in algorithms:
                    self.encryptionAlgorithms = list(filter(lambda i: i != algorithm, self.encryptionAlgorithms))
            else:
                logging.warning('IKEv2IpsecProposals has no members.  Cannot remove encryptionAlgorithms.')
        elif action == 'clear':
            if 'encryptionAlgorithms' in self.__dict__:
                del self.encryptionAlgorithms
                logging.info('All encryptionAlgorithms removed from this IKEv2IpsecProposals object.')

    def hash(self, action, algorithms=[]):
        logging.debug("In hash() for IKEv2IpsecProposals class.")
        if action == 'add':
            for algorithm in algorithms:
                if 'integrityAlgorithms' in self.__dict__:
                    if algorithm in self.integrityAlgorithms:
                        logging.warning(f'integrityAlgorithms "{algorithm}" already exists.')
                    elif algorithm in self.VALID_FOR_HASH:
                        self.integrityAlgorithms.append(algorithm)
                    else:
                        logging.warning(f'integrityAlgorithms "{algorithm}" not a valid type.')
                else:
                    self.integrityAlgorithms = [algorithm]
        elif action == 'remove':
            if 'integrityAlgorithms' in self.__dict__:
                for algorithm in algorithms:
                    self.integrityAlgorithms = list(filter(lambda i: i != algorithm, self.integrityAlgorithms))
            else:
                logging.warning('IKEv2IpsecProposals has no members.  Cannot remove integrityAlgorithms.')
        elif action == 'clear':
            if 'integrityAlgorithms' in self.__dict__:
                del self.integrityAlgorithms
                logging.info('All integrityAlgorithms removed from this IKEv2IpsecProposals object.')



import logging
import fmcapi
import time


def test__vlan_tag(fmc):
    logging.info('Test VlanTag.  Post, get, put, delete VLAN Tag Objects.')

    starttime = str(int(time.time()))
    namer = f'_fmcapi_test_{starttime}'

    obj1 = fmcapi.VlanTags(fmc=fmc)
    obj1.name = namer
    obj1.vlans(start_vlan='100', end_vlan='200')
    obj1.post()
    time.sleep(1)
    del obj1
    obj1 = fmcapi.VlanTags(fmc=fmc, name=namer)
    obj1.get()
    obj1.vlans(start_vlan='400', end_vlan='300')
    obj1.put()
    time.sleep(1)
    obj1.delete()

    logging.info('Test VlanTag done.\n')



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
import logging
import warnings


class ApplicationTypes(APIClassTemplate):
    """
    The ApplicationTypes Object in the FMC.
    """

    VALID_JSON_DATA = ['id', 'name', 'type']
    VALID_FOR_KWARGS = VALID_JSON_DATA + []
    URL_SUFFIX = '/object/applicationtypes'

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for ApplicationTypes class.")
        self.parse_kwargs(**kwargs)

    def post(self):
        logging.info('POST method for API for ApplicationTypes not supported.')
        pass

    def put(self):
        logging.info('PUT method for API for ApplicationTypes not supported.')
        pass

    def delete(self):
        logging.info('DELETE method for API for ApplicationTypes not supported.')
        pass


class ApplicationType(ApplicationTypes):
    """Dispose of this Class after 20210101."""

    def __init__(self, fmc, **kwargs):
        warnings.resetwarnings()
        warnings.warn("Deprecated: ApplicationType() should be called via ApplicationTypes().")
        super().__init__(fmc, **kwargs)



import logging
import fmcapi
import time


def test__devicegrouprecords(fmc):
    logging.info('Test DeviceGroupRecords: '
                 'get, post, put, delete DeviceGroupRecords Objects requires registered device')

    starttime = str(int(time.time()))
    namer = f'_fmcapi_test_{starttime}'

    device_list = [{"name": "ftdv-HA", "type": "deviceHAPair"}]
    dg1 = fmcapi.DeviceGroupRecords(fmc=fmc)
    dg1.name = "_dg1" + namer
    dg1.devices(action='add', members=device_list)
    dg1.post()
    time.sleep(1)

    dg1.get()
    dg1.devices(action='remove', members=device_list)
    dg1.put()
    time.sleep(1)

    dg1.get()
    dg1.devices(action='add', members=device_list)
    dg1.put()
    time.sleep(1)

    dg1.get()
    dg1.devices(action='clear')
    dg1.put()
    time.sleep(1)

    dg1.get()
    dg1.delete()
    logging.info('Testing DeviceGroupRecords class done.\n')



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
from .upgradepackages import UpgradePackages
import logging
import warnings


class ListApplicableDevices(APIClassTemplate):
    """
    The ListApplicableDevices Object in the FMC.
    """

    VALID_JSON_DATA = ['id', 'name', 'type', 'upgradePackage', 'model', 'modelId', 'modelNumber', 'modelType',
                       'healthStatus', 'sw_version', 'isPartofContainer', 'containerType', 'healthPolicy',
                       'accessPolicy',
                       ]
    VALID_FOR_KWARGS = VALID_JSON_DATA + []
    URL_SUFFIX = '/updates/upgradepackages'

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for ListApplicableDevices class.")
        self.type = 'UpgradePackage'
        self.parse_kwargs(**kwargs)

    def upgrade_package(self, package_name):
        logging.debug("In upgrade_package() for ListApplicableDevices class.")
        package1 = UpgradePackages(fmc=self.fmc)
        package1.get(name=package_name)
        if 'id' in package1.__dict__:
            self.package_id = package1.id
            self.URL = f'{self.fmc.platform_url}{self.URL_SUFFIX}/{self.package_id}/applicabledevices'
            self.package_added_to_url = True
        else:
            logging.warning(f'UpgradePackage {package_name} not found.  Cannot get list of ListApplicableDevices.')

    def post(self):
        logging.info('POST method for API for ListApplicableDevices not supported.')
        pass

    def put(self):
        logging.info('PUT method for API for ListApplicableDevices not supported.')
        pass

    def delete(self):
        logging.info('DELETE method for API for ListApplicableDevices not supported.')
        pass


class ApplicableDevices(ListApplicableDevices):
    """Dispose of this Class after 20210101."""

    def __init__(self, fmc, **kwargs):
        warnings.resetwarnings()
        warnings.warn("Deprecated: ApplicableDevices() should be called via ListApplicableDevices().")
        super().__init__(fmc, **kwargs)



import logging
import fmcapi


def test__failoverinterfacemacaddressconfigs(fmc):
    logging.info('Test FailoverInterfaceMACAddressConfigs. get, post, put, '
                 'delete FailoverInterfaceMACAddressConfigs Objects')

    obj1 = fmcapi.DeviceHAFailoverMAC(fmc=fmc, ha_name="HaName")
    obj1.p_interface(name="GigabitEthernet0/0", device_name="device_name")
    obj1.failoverActiveMac = "0050.5686.718f"
    obj1.failoverStandbyMac = "1050.5686.0c2e"
    logging.info('DeviceHAFailoverMAC POST->')
    logging.info(obj1.format_data())

    obj1.post()
    del obj1

    obj1 = fmcapi.DeviceHAFailoverMAC(fmc=fmc)
    obj1.edit(name="GigabitEthernet0/0", ha_name="HaName")
    obj1.failoverStandbyMac = "0050.5686.0c2e"
    logging.info('DeviceHAFailoverMAC PUT->')

    logging.info(obj1.format_data())
    del obj1

    obj1 = fmcapi.DeviceHAFailoverMAC(fmc=fmc)
    obj1.edit(name="GigabitEthernet0/0", ha_name="HaName")
    obj1.delete()



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
from fmcapi.api_objects.policy_services.accesspolicies import AccessPolicies
import time
import logging
import warnings


class DeviceRecords(APIClassTemplate):
    """
    The DeviceRecords Object in the FMC.
    """

    VALID_JSON_DATA = ['id', 'name', 'type', 'hostName', 'natID', 'regKey', 'license_caps', 'accessPolicy']
    VALID_FOR_KWARGS = VALID_JSON_DATA + ['acp_name', 'acp_id', 'model', 'modelId', 'modelNumber', 'modelType',
                                          'healthStatus', 'healthPolicy', 'keepLocalEvents', 'prohibitPacketTransfer',
                                          ]
    URL_SUFFIX = '/devices/devicerecords'
    REQUIRED_FOR_POST = ['accessPolicy', 'hostName', 'regKey']
    REQUIRED_FOR_PUT = ['id']
    LICENSES = ['BASE', 'MALWARE', 'URLFilter', 'THREAT', 'VPN']

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for DeviceRecords class.")
        self.parse_kwargs(**kwargs)

    def parse_kwargs(self, **kwargs):
        super().parse_kwargs(**kwargs)
        logging.debug("In parse_kwargs() for DeviceRecords class.")
        if 'acp_name' in kwargs:
            self.acp(name=kwargs['acp_name'])

    def licensing(self, action, name='BASE'):
        logging.debug("In licensing() for DeviceRecords class.")
        if action == 'add':
            if name in self.LICENSES:
                if 'license_caps' in self.__dict__:
                    self.license_caps.append(name)
                    self.license_caps = list(set(self.license_caps))
                else:
                    self.license_caps = [name]
                logging.info(f'License "{name}" added to this DeviceRecords object.')

            else:
                logging.warning(f'{name} not found in {self.LICENSES}.  Cannot add license to DeviceRecords.')
        elif action == 'remove':
            if name in self.LICENSES:
                if 'license_caps' in self.__dict__:
                    try:
                        self.license_caps.remove(name)
                    except ValueError:
                        logging.warning(f'{name} is not assigned to this devicerecord thus cannot be removed.')
                    logging.info(f'License "{name}" removed from this DeviceRecords object.')
                else:
                    logging.warning(f'{name} is not assigned to this devicerecord thus cannot be removed.')

            else:
                logging.warning(f'{name} not found in {self.LICENSES}.  Cannot remove license from DeviceRecords.')
        elif action == 'clear':
            if 'license_caps' in self.__dict__:
                del self.license_caps
                logging.info('All licensing removed from this DeviceRecords object.')

    def acp(self, name=''):
        logging.debug("In acp() for DeviceRecords class.")
        acp = AccessPolicies(fmc=self.fmc)
        acp.get(name=name)
        if 'id' in acp.__dict__:
            self.accessPolicy = {'id': acp.id, 'type': acp.type}
        else:
            logging.warning(f'Access Control Policy {name} not found.  Cannot set up accessPolicy for DeviceRecords.')

    def post(self, **kwargs):
        logging.debug("In post() for DeviceRecords class.")
        response = super().post(**kwargs)
        if 'post_wait_time' in kwargs:
            self.post_wait_time = kwargs['post_wait_time']
        else:
            self.post_wait_time = 300
        logging.info(f'DeviceRecords registration task submitted.  '
                     f'Waiting {self.post_wait_time} seconds for it to complete.')
        time.sleep(self.post_wait_time)
        return response


class Device(DeviceRecords):
    """Dispose of this Class after 20210101."""

    def __init__(self, fmc, **kwargs):
        warnings.resetwarnings()
        warnings.warn("Deprecated: Device() should be called via DeviceRecords().")
        super().__init__(fmc, **kwargs)



import logging
import fmcapi


def test__staticroutes(fmc):
    logging.info('Testing StaticRoutes class. Requires a registered device')

    obj1 = fmcapi.StaticRoutes(fmc=fmc)
    obj1.device(device_name="device_name")
    logging.info('All StaticRoutes -- >')
    result = obj1.get()
    logging.info(result)
    logging.info(f"Total items: {len(result['items'])}")

    del obj1

    logging.info('Testing StaticRoutes class done.\n')



import logging
import fmcapi


def test__intrusion_policy(fmc):
    logging.info('Test IntrusionPolicy. Can only GET IntrusionPolicy objects.')
    obj1 = fmcapi.IntrusionPolicies(fmc=fmc)
    obj1.get(name='Security Over Connectivity')
    logging.info('IntrusionPolicy -->')
    logging.info(obj1.format_data())

    logging.info('Test IntrusionPolicy done.\n')



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
import logging
import warnings


class UpgradePackages(APIClassTemplate):
    """
    The UpgradePackages Object in the FMC.
    """

    VALID_JSON_DATA = ['id', 'name', 'type']
    VALID_FOR_KWARGS = VALID_JSON_DATA + []
    URL_SUFFIX = '/updates/upgradepackages'

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for UpgradePackages class.")
        self.type = 'UpgradePackage'
        self.URL = f'{self.fmc.platform_url}{self.URL_SUFFIX}'
        self.parse_kwargs(**kwargs)

    def post(self):
        logging.info('POST method for API for UpgradePackages not supported.')
        pass

    def put(self):
        logging.info('PUT method for API for UpgradePackages not supported.')
        pass


class UpgradePackage(UpgradePackages):
    """Dispose of this Class after 20210101."""

    def __init__(self, fmc, **kwargs):
        warnings.resetwarnings()
        warnings.warn("Deprecated: UpgradePackage() should be called via UpgradePackages().")
        super().__init__(fmc, **kwargs)



import logging
import fmcapi
import time


def test__interface_group(fmc):
    logging.info('Test InterfaceGroup.  Post, get, put, delete InterfaceGroup Objects.')

    obj1 = fmcapi.InterfaceGroups(fmc=fmc)
    obj1.name = "_ig_outside_all"
    obj1.interfaceMode = 'ROUTED'
    logging.info('InterfaceGroup POST-->')
    logging.info(obj1.format_data())
    logging.info('\n')
    obj1.post()
    time.sleep(1)
    del obj1

    obj1 = fmcapi.InterfaceGroups(fmc=fmc, name="_ig_outside_all")
    obj1.get()
    obj1.p_interface(device_name="device_name",
                     action="add",
                     names=["GigabitEthernet0/0", "GigabitEthernet0/1", "GigabitEthernet0/2"])
    logging.info('InterfaceGroup PUT-->')
    logging.info(obj1.format_data())
    logging.info('\n')
    obj1.put()
    time.sleep(1)
    del obj1

    obj1 = fmcapi.InterfaceGroups(fmc=fmc, name="_ig_outside_all")
    obj1.get()
    obj1.p_interface(device_name="device_name",
                     action="remove",
                     names=["GigabitEthernet0/1"])
    logging.info('InterfaceGroup PUT-->')
    logging.info(obj1.format_data())
    logging.info('\n')
    obj1.put()
    time.sleep(1)
    del obj1

    obj1 = fmcapi.InterfaceGroups(fmc=fmc, name="_ig_outside_all")
    obj1.get()
    obj1.p_interface(action="clear-all")
    obj1.put()
    logging.info('InterfaceGroup DELETE-->')
    logging.info(obj1.format_data())
    logging.info('\n')
    obj1.delete()
    del obj1

    logging.info('Test InterfaceGroup done.\n')



import logging
import fmcapi


def test__application_productivity(fmc):
    logging.info('Testing ApplicationProductivity class.')

    obj1 = fmcapi.ApplicationProductivities(fmc=fmc)
    logging.info('All ApplicationProductivities -- >')
    result = obj1.get()
    logging.info(result)
    logging.info(f"Total items: {len(result['items'])}")
    logging.info('\n')
    del obj1
    obj1 = fmcapi.ApplicationProductivities(fmc=fmc, name='Very Low')
    logging.info('One ApplicationProductivity -- >')
    logging.info(obj1.get())

    logging.info('Testing ApplicationProductivity class done.\n')



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
from .securityzones import SecurityZones
import logging
import warnings


class SLAMonitors(APIClassTemplate):
    """
    The SLAMonitors Object in the FMC.
    """
    VALID_JSON_DATA = ['id', 'name', 'type', 'timeout', 'threshold', 'frequency', 'slaId', 'dataSize', 'tos',
                       'noOfPackets', 'monitorAddress', 'interfaceObjects', 'description'
                       ]
    VALID_FOR_KWARGS = VALID_JSON_DATA + []
    URL_SUFFIX = '/object/slamonitors'
    REQUIRED_FOR_POST = ['name', 'slaId', 'monitorAddress', 'interfaceObjects', 'type']
    REQUIRED_FOR_PUT = ['id', 'type']

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for SLAMonitors class.")
        self.parse_kwargs(**kwargs)
        self.type = "SLAMonitor"

    def interfaces(self, names):
        logging.debug("In interfaces() for SLAMonitors class.")
        zones = []
        for name in names:
            # Supports passing list of str
            sz = SecurityZones(fmc=self.fmc)
            sz.get(name=name)
            if 'id' in sz.__dict__:
                zones.append({'name': sz.name, 'id': sz.id, 'type': sz.type})
            else:
                logging.warning(f'Security Zone, "{name}", not found.  Cannot add to SLAMonitors.')
        if len(zones) != 0:
            # Make sure we found at least one zone
            self.interfaceObjects = zones
        else:
            logging.warning(f'No valid Security Zones found: "{names}".  Cannot add to SLAMonitosr.')


class SLAMonitor(SLAMonitors):
    """Dispose of this Class after 20210101."""

    def __init__(self, fmc, **kwargs):
        warnings.resetwarnings()
        warnings.warn("Deprecated: SLAMonitor() should be called via SLAMonitors().")
        super().__init__(fmc, **kwargs)



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
import logging


class Geolocation(APIClassTemplate):
    """
    The Geolocation Object in the FMC.
    """

    VALID_JSON_DATA = ['id', 'name', 'type', 'continentId', 'continents', 'countries', 'continentUUID']
    VALID_FOR_KWARGS = VALID_JSON_DATA + []
    URL_SUFFIX = '/object/geolocations'

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for Geolocation class.")
        self.parse_kwargs(**kwargs)

    def post(self):
        logging.info('POST method for API for Geolocation not supported.')
        pass

    def put(self):
        logging.info('PUT method for API for Geolocation not supported.')
        pass

    def delete(self):
        logging.info('DELETE method for API for Geolocation not supported.')
        pass



import logging
import fmcapi


def test__ports(fmc):
    logging.info('Test Ports.  This only returns a full list of various Port object types.')
    
    obj1 = fmcapi.Ports(fmc=fmc)
    logging.info('Ports -->')
    result = obj1.get()
    logging.info(result)

    logging.info('Test Ports done.\n')




import logging
import fmcapi
import time


def test__network_group(fmc):
    logging.info('Testing NetworkGroup class.')

    starttime = str(int(time.time()))
    namer = f'_fmcapi_test_{starttime}'

    obj10 = fmcapi.Hosts(fmc=fmc, name='_iphost1', value='3.3.3.3')
    obj10.post()
    obj11 = fmcapi.Networks(fmc=fmc, name='_ipnet1', value='3.3.3.0/24')
    obj11.post()
    obj12 = fmcapi.Ranges(fmc=fmc, name='_iprange1', value='3.3.3.3-33.33.33.33')
    obj12.post()
    time.sleep(1)

    obj1 = fmcapi.NetworkGroups(fmc=fmc, name=namer)
    obj1.named_networks(action='add', name=obj10.name)
    obj1.named_networks(action='add', name=obj10.name)
    obj1.named_networks(action='remove', name=obj10.name)
    obj1.named_networks(action='clear')
    obj1.named_networks(action='add', name=obj11.name)
    obj1.named_networks(action='add', name=obj12.name)
    obj1.named_networks(action='remove', name=obj11.name)
    obj1.post()
    time.sleep(1)
    del obj1

    obj1 = fmcapi.NetworkGroups(fmc=fmc, name=namer)
    obj1.get()
    obj1.unnamed_networks(action='add', value='1.2.3.4')
    obj1.unnamed_networks(action='clear')
    obj1.unnamed_networks(action='add', value='1.2.3.4')
    obj1.unnamed_networks(action='remove', value='1.2.3.4')
    obj1.unnamed_networks(action='add', value='6.7.8.9')
    obj1.unnamed_networks(action='add', value='1.2.3.0/24')
    obj1.post()

    time.sleep(1)
    obj1.delete()
    obj10.delete()
    obj11.delete()
    obj12.delete()

    logging.info('Testing NetworkGroup class done.\n')



import logging
from .policy_services.accesspolicies import AccessPolicies  # Needs loaded before Device
from .policy_services.accesspolicies import AccessControlPolicy  # Needs loaded before Device
from .device_services.devicerecords import DeviceRecords  # Needs loaded early.
from .device_services.devicerecords import Device  # Needs loaded early.

from .device_ha_pair_services import FTDDeviceHAPairs
from .device_ha_pair_services.ftddevicehapairs import DeviceHAPairs


from .device_group_services import DeviceGroupRecords
from .device_group_services import DeviceGroups

from .device_clusters import FTDDeviceCluster

from .device_ha_pair_services.failoverinterfacemacaddressconfigs import FailoverInterfaceMACAddressConfigs
from .device_ha_pair_services.failoverinterfacemacaddressconfigs import DeviceHAFailoverMAC
from .device_ha_pair_services.monitoredinterfaces import DeviceHAMonitoredInterfaces
from .device_ha_pair_services.monitoredinterfaces import MonitoredInterfaces

from .device_services.bridgegroupinterfaces import BridgeGroupInterfaces
from .device_services.etherchannelinterfaces import EtherchannelInterfaces
from .device_services.ipv4staticroutes import IPv4StaticRoutes
from .device_services.ipv4staticroutes import IPv4StaticRoute
from .device_services.ipv6staticroutes import IPv6StaticRoutes
from .device_services.ipv6staticroutes import IPv6StaticRoute
from .device_services.physicalinterfaces import PhysicalInterfaces
from .device_services.physicalinterfaces import PhysicalInterface
from .device_services.redundantinterfaces import RedundantInterfaces
from .device_services.staticroutes import StaticRoutes
from .device_services.subinterfaces import SubInterfaces

from .object_services.anyprotocolportobjects import AnyProtocolPortObjects
from .object_services.applications import Applications
from .object_services.applications import Application
from .object_services.applicationcategories import ApplicationCategories
from .object_services.applicationcategories import ApplicationCategory
from .object_services.applicationfilters import ApplicationFilters
from .object_services.applicationfilters import ApplicationFilter
from .object_services.applicationproductivities import ApplicationProductivities
from .object_services.applicationproductivities import ApplicationProductivity
from .object_services.applicationrisks import ApplicationRisks
from .object_services.applicationrisks import ApplicationRisk
from .object_services.applicationtags import ApplicationTags
from .object_services.applicationtags import ApplicationTag
from .object_services.applicationtypes import ApplicationTypes
from .object_services.applicationtypes import ApplicationType
from .object_services.certenrollments import CertEnrollments
from .object_services.certenrollments import CertEnrollment
from .object_services.continents import Continents
from .object_services.continents import Continent
from .object_services.countries import Countries
from .object_services.countries import Country
from .object_services.dnsservergroups import DNSServerGroups
from .object_services.endpointdevicetypes import EndPointDeviceTypes
from .object_services.extendedaccesslist import ExtendedAccessList
from .object_services.fqdns import FQDNS
from .object_services.geolocation import Geolocation
from .object_services.icmpv4objects import ICMPv4Objects
from .object_services.icmpv4objects import ICMPv4Object
from .object_services.icmpv6objects import ICMPv6Objects
from .object_services.icmpv6objects import ICMPv6Object
from .object_services.ikev1ipsecproposals import IKEv1IpsecProposals
from .object_services.ikev1policies import IKEv1Policies
from .object_services.ikev2ipsecproposals import IKEv2IpsecProposals
from .object_services.ikev2policies import IKEv2Policies
from .object_services.interfacegroups import InterfaceGroups
from .object_services.interfacegroups import InterfaceGroup
from .object_services.interfaceobjects import InterfaceObjects
from .object_services.interfaceobjects import InterfaceObject
from .object_services.networkaddresses import NetworkAddresses
from .object_services.networkaddresses import IPAddresses
from .object_services.hosts import Hosts
from .object_services.hosts import IPHost
from .object_services.networks import Networks
from .object_services.networks import IPNetwork
from .object_services.ranges import Ranges
from .object_services.ranges import IPRange
from .object_services.isesecuritygrouptags import ISESecurityGroupTags
from .object_services.networkgroups import NetworkGroups
from .object_services.networkgroups import NetworkGroup
from .object_services.portobjectgroups import PortObjectGroups
from .object_services.portobjectgroups import PortObjectGroup
from .object_services.ports import Ports
from .object_services.protocolportobjects import ProtocolPortObjects
from .object_services.protocolportobjects import ProtocolPort
from .object_services.realms import Realms
from .object_services.realmusergroups import RealmUserGroups
from .object_services.realmusers import RealmUsers
from .object_services.securitygrouptags import SecurityGroupTags
from .object_services.securityzones import SecurityZones
from .object_services.securityzones import SecurityZone
from .object_services.siurlfeeds import SIUrlFeeds
from .object_services.siurllists import SIUrlLists
from .object_services.slamonitors import SLAMonitors
from .object_services.slamonitors import SLAMonitor
from .object_services.tunneltags import TunnelTags
from .object_services.urls import URLs
from .object_services.urls import URL
from .object_services.urlcategories import URLCategories
from .object_services.urlcategories import URLCategory
from .object_services.urlgroups import URLGroups
from .object_services.urlgroups import URLGroup
from .object_services.variablesets import VariableSets
from .object_services.variablesets import VariableSet
from .object_services.vlangrouptags import VlanGroupTags
from .object_services.vlangrouptags import VlanGroupTag
from .object_services.vlantags import VlanTags
from .object_services.vlantags import VlanTag

from .policy_services.accessrules import AccessRules
from .policy_services.accessrules import ACPRule
from .policy_services.accessrules import Bulk
from .policy_services.autonatrules import AutoNatRules
from .policy_services.filepolicies import FilePolicies
from .policy_services.ftdnatpolicies import FTDNatPolicies
from .policy_services.ftdnatpolicies import FTDNatPolicy
from .policy_services.hitcounts import HitCounts
from .policy_services.hitcounts import HitCount
from .policy_services.intrusionpolicies import IntrusionPolicies
from .policy_services.intrusionpolicies import IntrusionPolicy
from .policy_services.manualnatrules import ManualNatRules
from .policy_services.natrules import NatRules
from .policy_services.prefilterpolicies import PreFilterPolicies
from .policy_services.prefilterpolicies import PreFilterPolicy
from .policy_services.defaultactions import DefaultActions

from .policy_assignment_services.policyassignments import PolicyAssignments

from .status_services.taskstatuses import TaskStatuses

from .update_packages.listapplicabledevices import ListApplicableDevices
from .update_packages.listapplicabledevices import ApplicableDevices
from .update_packages.upgradepackages import UpgradePackages
from .update_packages.upgradepackages import UpgradePackage
from .update_packages.upgradepackage import Upgrades

logging.debug("In the api_objects __init__.py file.")

__all__ = [
    'DefaultActions',
    'DeviceGroupRecords',
    'DeviceGroups',
    'FTDDeviceCluster',
    'FTDDeviceHAPairs',
    'DeviceHAPairs',
    'FailoverInterfaceMACAddressConfigs',
    'DeviceHAFailoverMAC',
    'MonitoredInterfaces',
    'DeviceHAMonitoredInterfaces',
    'Device',
    'DeviceRecords',
    'StaticRoutes',
    'IPv4StaticRoutes',
    'IPv4StaticRoute',
    'IPv6StaticRoutes',
    'IPv6StaticRoute',
    'PhysicalInterfaces',
    'PhysicalInterface',
    'BridgeGroupInterfaces',
    'RedundantInterfaces',
    'EtherchannelInterfaces',
    'SubInterfaces',
    'AnyProtocolPortObjects',
    'ApplicationCategories',
    'ApplicationCategory',
    'ApplicationFilters',
    'ApplicationFilter',
    'ApplicationProductivities',
    'ApplicationProductivity',
    'ApplicationRisks',
    'ApplicationRisk',
    'Applications',
    'Application',
    'ApplicationTags',
    'ApplicationTag',
    'ApplicationTypes',
    'ApplicationType',
    'CertEnrollments',
    'CertEnrollment',
    'Continents',
    'Continent',
    'Countries',
    'Country',
    'DNSServerGroups',
    'EndPointDeviceTypes',
    'ExtendedAccessList',
    'FQDNS',
    'Geolocation',
    'Hosts',
    'IPHost',
    'ICMPv4Objects',
    'ICMPv4Object',
    'ICMPv6Objects',
    'ICMPv6Object',
    'IKEv1IpsecProposals',
    'IKEv1Policies',
    'IKEv2IpsecProposals',
    'IKEv2Policies',
    'InterfaceGroups',
    'InterfaceGroup',
    'InterfaceObjects',
    'InterfaceObject',
    'ISESecurityGroupTags',
    'NetworkAddresses',
    'IPAddresses',
    'NetworkGroups',
    'NetworkGroup',
    'Networks',
    'IPNetwork',
    'PortObjectGroups',
    'PortObjectGroup',
    'Ports',
    'ProtocolPortObjects',
    'ProtocolPort',
    'Ranges',
    'IPRange',
    'Realms',
    'RealmUserGroups',
    'RealmUsers',
    'SecurityGroupTags',
    'SecurityZones',
    'SecurityZone',
    'SIUrlFeeds',
    'SIUrlLists',
    'SLAMonitors',
    'SLAMonitor',
    'TunnelTags',
    'URLCategories',
    'URLCategory',
    'URLGroups',
    'URLGroup',
    'URLs',
    'URL',
    'VariableSets',
    'VariableSet',
    'VlanGroupTags',
    'VlanGroupTag',
    'VlanTags',
    'VlanTag',
    'PolicyAssignments',
    'AccessPolicies',
    'AccessControlPolicy',
    'AccessRules',
    'ACPRule',
    'Bulk',
    'FilePolicies',
    'FTDNatPolicies',
    'FTDNatPolicy',
    'AutoNatRules',
    'ManualNatRules',
    'NatRules',
    'IntrusionPolicies',
    'IntrusionPolicy',
    'PreFilterPolicies',
    'PreFilterPolicy',
    'HitCounts',
    'HitCount',
    'TaskStatuses',
    'ListApplicableDevices',
    'ApplicableDevices',
    'UpgradePackages',
    'UpgradePackage',
    'Upgrades',
]



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
from .ftdnatpolicies import FTDNatPolicies
import logging


class NatRules(APIClassTemplate):
    """
    The NatRules Object in the FMC.
    """

    VALID_JSON_DATA = ['id', 'name', 'type']
    VALID_FOR_KWARGS = VALID_JSON_DATA + []
    PREFIX_URL = '/policy/ftdnatpolicies'
    VALID_CHARACTERS_FOR_NAME = """[.\w\d_\- ]"""

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for NatRules class.")
        self.parse_kwargs(**kwargs)

    def nat_policy(self,name):
        logging.debug("In nat_policy() for NatRules class.")
        ftd_nat = FTDNatPolicies(fmc=self.fmc)
        ftd_nat.get(name=name)
        if 'id' in ftd_nat.__dict__:
            self.nat_id = ftd_nat.id
            self.URL = f'{self.fmc.configuration_url}{self.PREFIX_URL}/{self.nat_id}/natrules'
            self.nat_added_to_url = True
        else:
            logging.warning(f'FTD NAT Policy {name} not found.  Cannot set up NatRules for NAT Policy.')

    def post(self):
        logging.info('POST method for API for NatRules not supported.')
        pass

    def put(self):
        logging.info('PUT method for API for NatRules not supported.')
        pass

    def delete(self):
        logging.info('DELETE method for API for NatRules not supported.')
        pass



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
from fmcapi.api_objects.helper_functions import validate_vlans
import logging
import warnings


class VlanTags(APIClassTemplate):
    """
    The VlanTags Object in the FMC.
    """

    VALID_JSON_DATA = ['id', 'name', 'type', 'data', 'description']
    VALID_FOR_KWARGS = VALID_JSON_DATA + []
    URL_SUFFIX = '/object/vlantags'
    REQUIRED_FOR_POST = ['name', 'data']

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for VlanTags class.")
        self.type = 'VlanTag'
        self.parse_kwargs(**kwargs)

    def vlans(self, start_vlan, end_vlan=''):
        logging.debug("In vlans() for VlanTags class.")
        start_vlan, end_vlan = validate_vlans(start_vlan=start_vlan, end_vlan=end_vlan)
        self.data = {'startTag': start_vlan, 'endTag': end_vlan}


class VlanTag(VlanTags):
    """Dispose of this Class after 20210101."""

    def __init__(self, fmc, **kwargs):
        warnings.resetwarnings()
        warnings.warn("Deprecated: VlanTag() should be called via VlanTags().")
        super().__init__(fmc, **kwargs)



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
from fmcapi.api_objects.helper_functions import *
from .networkaddresses import NetworkAddresses
import logging
import warnings


class NetworkGroups(APIClassTemplate):
    """
    The NetworkGroups Object in the FMC.
    """

    VALID_JSON_DATA = ['id', 'name', 'type', 'objects', 'literals']
    VALID_FOR_KWARGS = VALID_JSON_DATA + []
    URL_SUFFIX = '/object/networkgroups'

    # Technically you can have objects OR literals but I'm not set up for "OR" logic, yet.
    REQUIRED_FOR_POST = ['name']

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for NetworkGroups class.")
        self.parse_kwargs(**kwargs)
        self.type = 'NetworkGroup'

    def named_networks(self, action, name=''):
        logging.debug("In named_networks() for NetworkGroups class.")
        if action == 'add':
            net1 = NetworkAddresses(fmc=self.fmc)
            response = net1.get()
            if 'items' in response:
                new_net = None
                for item in response['items']:
                    if item['name'] == name:
                        new_net = {'name': item['name'], 'id': item['id'], 'type': item['type']}
                        break
                if new_net is None:
                    logging.warning(f'Network "{name}" is not found in FMC.  Cannot add to NetworkGroups.')
                else:
                    if 'objects' in self.__dict__:
                        duplicate = False
                        for obj in self.objects:
                            if obj['name'] == new_net['name']:
                                duplicate = True
                                break
                        if not duplicate:
                            self.objects.append(new_net)
                            logging.info(f'Adding "{name}" to NetworkGroups.')
                    else:
                        self.objects = [new_net]
                        logging.info(f'Adding "{name}" to NetworkGroups.')
        if action == 'addgroup':
            netg1 = NetworkGroups(fmc=self.fmc)
            response = netg1.get()
            if 'items' in response:
                new_net = None
                for item in response['items']:
                    if item['name'] == name:
                        new_net = {'name': item['name'], 'id': item['id'], 'type': item['type']}
                        break
                if new_net is None:
                    logging.warning(f'Network "{name}" is not found in FMC.  Cannot add to NetworkGroups.')
                else:
                    if 'objects' in self.__dict__:
                        duplicate = False
                        for obj in self.objects:
                            if obj['name'] == new_net['name']:
                                duplicate = True
                                break
                        if not duplicate:
                            self.objects.append(new_net)
                            logging.info(f'Adding "{name}" to NetworkGroups.')
                    else:
                        self.objects = [new_net]
                        logging.info(f'Adding "{name}" to NetworkGroups.')
        elif action == 'remove':
            if 'objects' in self.__dict__:
                objects_list = []
                for obj in self.objects:
                    if obj['name'] != name:
                        objects_list.append(obj)
                self.objects = objects_list
                logging.info(f'Removed "{name}" from NetworkGroups.')
            else:
                logging.info("This NetworkGroups has no named_networks.  Nothing to remove.")
        elif action == 'clear':
            if 'objects' in self.__dict__:
                del self.objects
                logging.info('All named_networks removed from this NetworkGroups.')

    def unnamed_networks(self, action, value=''):
        logging.debug("In unnamed_networks() for NetworkGroups class.")
        new_literal = []
        if action == 'add':
            if value == '':
                logging.error('Value assignment required to add unamed_network to NetworkGroups.')
                return
            literal_type = get_networkaddress_type(value=value)
            if literal_type == 'host' or literal_type == 'network':
                new_literal = {'value': value, 'type': literal_type}
            elif literal_type == 'range':
                logging.error('Ranges are not supported as unnamed_networks in a NetworkGroups.')
            else:
                logging.error(f'Value "{value}" provided is not in a recognizable format.')
                return
            if 'literals' in self.__dict__:
                duplicate = False
                for obj in self.literals:
                    if obj['value'] == new_literal['value']:
                        duplicate = True
                        break
                if not duplicate:
                    self.literals.append(new_literal)
                    logging.info(f'Adding "{value}" to NetworkGroup.')
            else:
                self.literals = [new_literal]
                logging.info(f'Adding "{value}" to NetworkGroup.')
        elif action == 'remove':
            if 'literals' in self.__dict__:
                literals_list = []
                for obj in self.literals:
                    if obj['value'] != value:
                        literals_list.append(obj)
                self.literals = literals_list
                logging.info(f'Removed "{value}" from NetworkGroup.')
            else:
                logging.info("This NetworkGroups has no unnamed_networks.  Nothing to remove.")
        elif action == 'clear':
            if 'literals' in self.__dict__:
                del self.literals
                logging.info('All unnamed_networks removed from this NetworkGroups.')


class NetworkGroup(NetworkGroups):
    """Dispose of this Class after 20210101."""

    def __init__(self, fmc, **kwargs):
        warnings.resetwarnings()
        warnings.warn("Deprecated: NetworkGroup() should be called via NetworkGroups().")
        super().__init__(fmc, **kwargs)




from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
import logging
import warnings


class AccessPolicies(APIClassTemplate):
    """
    The AccessPolicies Object in the FMC.
    """

    VALID_JSON_DATA = ['id', 'name', 'type', 'description', 'defaultAction']
    VALID_FOR_KWARGS = VALID_JSON_DATA + []
    URL_SUFFIX = '/policy/accesspolicies'
    REQUIRED_FOR_POST = ['name']
    DEFAULT_ACTION_OPTIONS = ['BLOCK', 'NETWORK_DISCOVERY', 'IPS']  # Not implemented yet.
    FILTER_BY_NAME = True

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for AccessPolicies class.")
        self.parse_kwargs(**kwargs)

    def parse_kwargs(self, **kwargs):
        super().parse_kwargs(**kwargs)
        logging.debug("In parse_kwargs() for AccessPolicies class.")
        if 'defaultAction' in kwargs:
            self.defaultAction = kwargs['defaultAction']
        else:
            self.defaultAction = {'action': 'BLOCK'}

    def put(self, **kwargs):
        logging.info('The put() method for the AccessPolicies() class can work but I need to write a '
                     'DefaultAction() class and accommodate for such before "putting".')
        pass


class AccessControlPolicy(AccessPolicies):
    """Dispose of this Class after 20210101."""

    def __init__(self, fmc, **kwargs):
        warnings.resetwarnings()
        warnings.warn("Deprecated: AccessControlPolicy() should be called via AccessPolicies().")
        super().__init__(fmc, **kwargs)



import logging
import fmcapi
import time


def test__phys_interfaces(fmc):
    logging.info('Test PhysicalInterface.  get, put PhysicalInterface Objects. Requires registered device')

    sz1 = fmcapi.SecurityZones(fmc=fmc)
    sz1.name = "SZ-OUTSIDE1"
    sz1.post()
    time.sleep(1)
    sz2 = fmcapi.SecurityZones(fmc=fmc)
    sz2.name = "SZ-OUTSIDE2"
    sz2.post()
    time.sleep(1)

    intf1 = fmcapi.PhysicalInterfaces(fmc=fmc, device_name="device_name")
    intf1.get(name="GigabitEthernet0/0")
    intf1.enabled = True
    intf1.ifname = "OUTSIDE1"
    intf1.activeMACAddress = "0050.5686.718f"
    intf1.standbyMACAddress = "0050.5686.0c2e"
    intf1.static(ipv4addr="10.254.0.3", ipv4mask=24)
    intf1.sz(name=sz1.name)
    intf2 = fmcapi.PhysicalInterfaces(fmc=fmc, device_name="device_name")
    intf2.get(name="GigabitEthernet0/1")
    intf2.enabled = True
    intf2.ifname = "OUTSIDE2"
    intf2.activeMACAddress = "0050.5686.821d"
    intf2.standbyMACAddress = "0050.5686.11cb"
    intf2.dhcp()
    intf2.sz(name=sz2.name)
    intf1.put()
    time.sleep(1)
    intf2.put()
    time.sleep(1)
    intf1.get()
    intf2.get()

    intf1.enabled = False
    intf1.activeMACAddress = ""
    intf1.standbyMACAddress = ""
    intf1.static(ipv4addr="", ipv4mask="")
    intf1.securityZone = {}
    intf1.activeMACAddress = ""
    intf1.standbyMACAddress = ""
    intf2.enabled = False
    intf2.activeMACAddress = ""
    intf2.standbyMACAddress = ""
    intf2.static(ipv4addr="", ipv4mask="")
    intf2.securityZone = {}
    intf2.activeMACAddress = ""
    intf2.standbyMACAddress = ""
    intf1.put()
    time.sleep(1)
    intf2.put()
    time.sleep(1)
    intf1.get()
    intf2.get()
    intf1.ifname = ""
    intf2.ifname = ""
    intf1.put()
    sz1.delete()
    intf2.put()
    sz2.delete()



import logging
import fmcapi
import time


def test__manualnat(fmc):
    logging.info('Testing ManualNatRules class.')

    starttime = str(int(time.time()))
    namer = f'_fmcapi_test_{starttime}'

    # Create a container policy for FTD NAT rules
    natpol1 = fmcapi.FTDNatPolicies(fmc=fmc, name=namer)
    natpol1.post()
    natpol1.get()

    # Create original and translate objects
    obj1 = fmcapi.Networks(fmc=fmc)
    obj1.name = '_net_original'
    obj1.value = '10.0.0.0/8'
    obj1.post()
    time.sleep(1)

    obj2 = fmcapi.Hosts(fmc=fmc)
    obj2.name = '_net_xlate'
    obj2.value = '192.0.2.1'
    obj2.post()
    time.sleep(1)

    # Create identity nat object
    obj3 = fmcapi.Networks(fmc=fmc)
    obj3.name = '_net_identity'
    obj3.value = '192.168.0.0/24'
    obj3.post()
    time.sleep(1)

    # Create nat pool objects
    obj4 = fmcapi.Networks(fmc=fmc)
    obj4.name = '_net_original_pool'
    obj4.value = '172.16.0.0/24'
    obj4.post()
    time.sleep(1)

    # PAT Pool must be a range, not a subnet
    obj5 = fmcapi.Ranges(fmc=fmc)
    obj5.name = '_net_xlate_pool'
    obj5.value = '192.0.2.128-192.0.2.254'
    obj5.post()
    time.sleep(1)

    # Create interface PAT object
    obj6 = fmcapi.Networks(fmc=fmc)
    obj6.name = '_net_original_intf'
    obj6.value = '192.168.1.0/24'
    obj6.post()

    # Create NAT divert objects
    obj7 = fmcapi.Hosts(fmc=fmc)
    obj7.name = '_net_source_divert'
    obj7.value = '172.30.1.1'
    obj7.post()
    time.sleep(1)

    obj8 = fmcapi.Hosts(fmc=fmc)
    obj8.name = '_net_destination_divert'
    obj8.value = '4.2.2.2'
    obj8.post()
    time.sleep(1)

    # Create port-based NAT objects
    obj9 = fmcapi.Hosts(fmc=fmc)
    obj9.name = '_net_source_portbased'
    obj9.value = '172.30.1.2'
    obj9.post()
    time.sleep(1)

    obj10 = fmcapi.Hosts(fmc=fmc)
    obj10.name = '_net_xlate_portbased'
    obj10.value = '192.0.2.254'
    obj10.post()
    time.sleep(1)

    obj11 = fmcapi.ProtocolPortObjects(fmc=fmc)
    obj11.name = '_port_original'
    obj11.protocol = 'TCP'
    obj11.port = '443'
    obj11.post()
    time.sleep(1)

    obj12 = fmcapi.ProtocolPortObjects(fmc=fmc)
    obj12.name = '_port_xlate'
    obj12.protocol = 'TCP'
    obj12.port = '8443'
    obj12.post()
    time.sleep(1)

    # Create Security Zones
    sz1 = fmcapi.SecurityZones(fmc=fmc, name='IG-INSIDE')
    sz1.post()
    sz2 = fmcapi.SecurityZones(fmc=fmc, name='SZ-OUTSIDE1')
    sz2.post()

    # Manualnat a network object to a host
    manualnat1 = fmcapi.ManualNatRules(fmc=fmc)
    manualnat1.original_source(name="_net_original")
    manualnat1.translated_source(name="_net_xlate")
    manualnat1.natType = "STATIC"
    # Source and destination interface can be either an interface group or security zone
    manualnat1.source_intf(name="IG-INSIDE")
    manualnat1.destination_intf(name="SZ-OUTSIDE1")
    manualnat1.enabled = True
    manualnat1.nat_policy(name=namer)

    # Manualnat identity nat
    manualnat2 = fmcapi.ManualNatRules(fmc=fmc)
    manualnat2.identity_nat(name="_net_identity")
    manualnat2.source_intf(name="IG-INSIDE")
    manualnat2.destination_intf(name="SZ-OUTSIDE1")
    manualnat2.enabled = True
    manualnat2.nat_policy(name=namer)

    # Manualnat nat pool
    manualnat3 = fmcapi.ManualNatRules(fmc=fmc)
    manualnat3.original_source(name="_net_original_pool")
    manualnat3.patPool(name="_net_xlate_pool")
    manualnat3.source_intf(name="IG-INSIDE")
    manualnat3.destination_intf(name="SZ-OUTSIDE1")
    manualnat3.enabled = True
    manualnat3.nat_policy(name=namer)

    # Manualnat interface PAT
    manualnat4 = fmcapi.ManualNatRules(fmc=fmc)
    manualnat4.original_source(name="_net_original_intf")
    manualnat4.natType = "DYNAMIC"
    manualnat4.unidirectional = True
    manualnat4.source_intf(name="IG-INSIDE")
    manualnat4.destination_intf(name="SZ-OUTSIDE1")
    manualnat4.nat_policy(name=namer)
    manualnat4.enabled = True
    manualnat4.interfaceInTranslatedSource = True

    # Manualnat divert
    manualnat5 = fmcapi.ManualNatRules(fmc=fmc)
    manualnat5.identity_nat(name="_net_source_divert")
    manualnat5.original_destination(name="_net_destination_divert")
    manualnat5.source_intf(name="IG-INSIDE")
    manualnat5.destination_intf(name="SZ-OUTSIDE1")
    manualnat5.enabled = True
    manualnat5.nat_policy(name=namer)

    # Manualnat port-based
    manualnat6 = fmcapi.ManualNatRules(fmc=fmc)
    manualnat6.original_source(name="_net_source_portbased")
    manualnat6.original_source_port(name="_port_original")
    manualnat6.translated_source(name="_net_xlate_portbased")
    manualnat6.translated_source_port(name="_port_xlate")
    manualnat6.natType = "STATIC"
    manualnat6.source_intf(name="IG-INSIDE")
    manualnat6.destination_intf(name="SZ-OUTSIDE1")
    manualnat6.enabled = True
    manualnat6.nat_policy(name=namer)

    manualnat1.post()
    manualnat2.post()
    manualnat3.post()
    manualnat4.post()
    manualnat5.post()
    manualnat6.post()

    '''
    # Associate a nat policy to a device
    # Do not uncomment if you do not have a device registered to FPMC
    # Use name of device or deviceHAPair as applicable
    pol_devices = [{"name": "ftdv-HA", "type": "deviceHAPair"}]
    assign1 = PolicyAssignments(fmc=fmc)
    assign1.ftd_natpolicy(name=namer, devices=pol_devices)
    assign1.post()

    assign1.ftd_natpolicy(name=namer, devices=[])
    assign1.put()
    '''

    logging.info('Cleanup of testing ManualNatRule methods.')

    natpol1.delete()
    obj1.delete()
    obj2.delete()
    obj3.delete()
    obj4.delete()
    obj5.delete()
    obj6.delete()
    obj7.delete()
    obj8.delete()
    obj9.delete()
    obj10.delete()
    obj11.delete()
    obj12.delete()
    sz1.delete()
    sz2.delete()



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
from fmcapi.api_objects.device_services.devicerecords import DeviceRecords
from fmcapi.api_objects.device_ha_pair_services.ftddevicehapairs import FTDDeviceHAPairs
import logging
import warnings


class DeviceGroupRecords(APIClassTemplate):
    """
    The DeviceGroupRecords Object in the FMC.
    """

    VALID_JSON_DATA = ['id', 'name', 'members']
    VALID_FOR_KWARGS = VALID_JSON_DATA + []
    URL_SUFFIX = '/devicegroups/devicegrouprecords'

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for DeviceGroupRecords class.")
        self.type = 'DeviceGroup'
        self.parse_kwargs(**kwargs)

    def devices(self, action, members=[]):
        logging.debug("In devices() for DeviceGroupRecords class.")
        if action == 'add':
            for member in members:
                if member["type"] == 'device':
                    dev1 = DeviceRecords(fmc=self.fmc)
                    dev1.get(name=member["name"])
                elif member["type"] == 'deviceHAPair':
                    dev1 = FTDDeviceHAPairs(fmc=self.fmc)
                    dev1.get(name=member["name"])
                if 'id' in dev1.__dict__:
                    if 'members' in self.__dict__:
                        self.members.append({"id": dev1.id, "type": dev1.type, "name": dev1.name})
                    else:
                        self.members = [{"id": dev1.id, "type": dev1.type, "name": dev1.name}]
                    logging.info(f'DeviceRecord "{dev1.name}" added to this DeviceGroupRecords object.')
                else:
                    logging.warning(f'{member} not found.  Cannot add DeviceRecord to DeviceGroupRecords.')
        elif action == 'remove':
            if 'members' in self.__dict__:
                for member in members:
                    if member["type"] == 'device':
                        dev1 = DeviceRecords(fmc=self.fmc)
                        dev1.get(name=member["name"])
                    elif member["type"] == 'deviceHAPair':
                        dev1 = FTDDeviceHAPairs(fmc=self.fmc)
                        dev1.get(name=member["name"])
                    if 'id' in dev1.__dict__:
                        if member["type"] == 'device':
                            self.members = list(filter(lambda i: i['id'] != dev1.id, self.members))
                        elif member["type"] == 'deviceHAPair':
                            devHA1 = FTDDeviceHAPairs(fmc=self.fmc)
                            devHA1.get(name=member["name"])
                            self.members = list(filter(lambda i: i['id'] != devHA1.primary["id"], self.members))
                            self.members = list(filter(lambda i: i['id'] != devHA1.secondary["id"], self.members))
                    else:
                        logging.warning(f'DeviceRecord {member} not registered.  Cannot remove DeviceRecord'
                                        f' from DeviceGroupRecords.')
            else:
                logging.warning('DeviceGroupRecords has no members.  Cannot remove DeviceRecord.')
        elif action == 'clear':
            if 'members' in self.__dict__:
                del self.members
                logging.info('All device records removed from this DeviceGroupRecords object.')


class DeviceGroups(DeviceGroupRecords):
    """Dispose of this Class after 20210101."""

    def __init__(self, fmc, **kwargs):
        warnings.resetwarnings()
        warnings.warn("Deprecated: DeviceGroups() should be called via DeviceGroupRecords().")
        super().__init__(fmc, **kwargs)



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
from fmcapi.api_objects.device_services.devicerecords import DeviceRecords
from fmcapi.api_objects.object_services.securityzones import SecurityZones
from fmcapi.api_objects.device_services.physicalinterfaces import PhysicalInterfaces
import logging


class SubInterfaces(APIClassTemplate):
    """
    The Subinterface Object in the FMC.
    """
    VALID_JSON_DATA = ['id', 'name', 'type', 'mode', 'enabled', 'MTU', 'managementOnly', 'ipAddress', 'subIntfId',
                       'vlanId', 'macLearn', 'ifname', 'securityZone', 'arpConfig', 'ipv4', 'ipv6', 'macTable',
                       'enableAntiSpoofing', 'fragmentReassembly', 'enableDNSLookup', 'activeMACAddress',
                       'standbyMACAddress',
                       ]
    VALID_FOR_KWARGS = VALID_JSON_DATA + ['device_name']
    VALID_CHARACTERS_FOR_NAME = """[.\w\d_\-\/\. ]"""
    PREFIX_URL = '/devices/devicerecords'
    URL_SUFFIX = None
    REQUIRED_FOR_POST = ['name', 'subIntfId', 'MTU']
    REQUIRED_FOR_PUT = ['id', 'device_id']
    VALID_FOR_IPV4 = ['static', 'dhcp', 'pppoe']
    VALID_FOR_MODE = ['INLINE', 'PASSIVE', 'TAP', 'ERSPAN', 'NONE']
    VALID_FOR_MTU = range(64, 9000)

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for SubInterfaces class.")
        self.parse_kwargs(**kwargs)
        self.type = "SubInterface"

    def parse_kwargs(self, **kwargs):
        super().parse_kwargs(**kwargs)
        logging.debug("In parse_kwargs() for SubInterfaces class.")
        if 'device_name' in kwargs:
            self.device(device_name=kwargs['device_name'])
        if 'ipv4' in kwargs:
            if list(kwargs['ipv4'].keys())[0] in self.VALID_FOR_IPV4:
                self.ipv4 = kwargs['ipv4']
            else:
                logging.warning(f"""Method "{kwargs['ipv4']}" is not a valid ipv4 type.""")
        if 'mode' in kwargs:
            if kwargs['mode'] in self.VALID_FOR_MODE:
                self.mode = kwargs['mode']
            else:
                logging.warning(f"""Mode "{kwargs['mode']}" is not a valid mode.""")
        if 'MTU' in kwargs:
            if kwargs['MTU'] in self.VALID_FOR_MTU:
                self.MTU = kwargs['MTU']
            else:
                logging.warning(f"""MTU "{kwargs['MTU']}" should be in the range 64-9000.""")
                self.MTU = 1500

    def device(self, device_name):
        logging.debug("In device() for SubInterfaces class.")
        device1 = DeviceRecords(fmc=self.fmc)
        device1.get(name=device_name)
        if 'id' in device1.__dict__:
            self.device_id = device1.id
            self.URL = f'{self.fmc.configuration_url}{self.PREFIX_URL}/{self.device_id}/subinterfaces'
            self.device_added_to_url = True
        else:
            logging.warning(f'Device "{device_name}" not found.  Cannot set up device for SubInterfaces.')

    def sz(self, name):
        logging.debug("In sz() for SubInterfaces class.")
        sz = SecurityZones(fmc=self.fmc)
        sz.get(name=name)
        if 'id' in sz.__dict__:
            new_zone = {'name': sz.name, 'id': sz.id, 'type': sz.type}
            self.securityZone = new_zone
        else:
            logging.warning(f'Security Zone, "{name}", not found.  Cannot add to SubInterfaces.')

    def static(self, ipv4addr, ipv4mask):
        logging.debug("In static() for SubInterfaces class.")
        self.ipv4 = {"static": {"address": ipv4addr, "netmask": ipv4mask}}

    def dhcp(self, enableDefault=True, routeMetric=1):
        logging.debug("In dhcp() for SubInterfaces class.")
        self.ipv4 = {"dhcp": {"enableDefaultRouteDHCP": enableDefault, "dhcpRouteMetric": routeMetric}}

    def p_interface(self, p_interface, device_name):
        logging.debug("In p_interface() for SubInterfaces class.")
        intf1 = PhysicalInterfaces(fmc=self.fmc)
        intf1.get(name=p_interface, device_name=device_name)
        if 'id' in intf1.__dict__:
            self.name = intf1.name
            if 'MTU' not in self.__dict__:
                self.MTU = intf1.MTU
        else:
            logging.warning(f'PhysicalInterface, "{intf1.name}", not found.  Cannot add to SubInterfaces.')
